/*
 * network.h
 *
 *  Created on: 05.01.2013
 *      Author: AndreR
 */

#ifndef NETWORK_H_
#define NETWORK_H_

#include "hal/eth.h"
#include "network/inet.h"
#include "commands.h"
#include "constants.h"
#include "util/nrf24_ex.h"
#include "util/flash_fs.h"
#include "ch.h"

#define NETWORK_MAX_FRAME_SIZE 1524
#define NETWORK_EVENT_QUEUE_SIZE 4

typedef int16_t(*UDPCallback)(uint8_t* pData, uint16_t dataLength);
typedef int16_t(*VisionCallback)(uint8_t* pData, uint16_t dataLength);

typedef struct _NetworkConfig
{
	struct _my
	{
		MAC mac;
		uint16_t port;
		IPv4Address ip;
	} my;

	struct _vision
	{
		IPv4Address ip;		// 224.5.23.2
		uint16_t port;		// 10002
	} vision;
} NetworkConfig;

typedef struct _Network
{
	uint8_t rxProcBuf[NETWORK_MAX_FRAME_SIZE];
	uint8_t txProcBuf[NETWORK_MAX_FRAME_SIZE];

	Mutex sendPacketMutex;
	Mailbox eventQueue;
	msg_t eventQueueData[NETWORK_EVENT_QUEUE_SIZE];

	uint16_t ethernetIdent;

	BaseStationEthStats bsEth;
	BaseStationWifiStats bsWifi;

	NetworkConfig cfg;	// own values used as source

	struct	// values used as unicast destination
	{
		MAC mac;
		IPv4Address ip;
		uint16_t port;
	} ucast;

	MAC visionMAC; // 01-00-5E-05-17-02

	struct
	{
		uint32_t respTime;
		uint8_t respCounter;
		uint32_t lastSendTime;
		uint8_t robustness;
	} igmp;

	FlashFile* pCfgFile;
} Network;

extern Network network;

void	NetworkInit();
msg_t	NetworkTask(void* params);
void	NetworkSetMyIP(IPv4Address ip);
void	NetworkSetMyPort(uint16_t port);
void	NetworkSetMyMAC(uint8_t inc);
int16_t	NetworkSendPacket(PacketHeader* pHeader, uint8_t* pData, uint16_t dataLength);
void	NetworkSetUDPCallback(UDPCallback cb);
void	NetworkSetVisionCallback(VisionCallback cb);
int16_t	NetworkSendIGMPv3MembershipReport(uint16_t numberOfGroupRecords, uint8_t* recordData, uint16_t recordDataLength);
void	NetworkSaveConfig();
void	NetworkSetVisionPort(uint16_t port);
void	NetworkSetVisionIP(IPv4Address ip);
void	NetworkPrintConfig();

#endif /* NETWORK_H_ */
