/*
 * vision.c
 *
 *  Created on: 22.05.2013
 *      Author: AndreR
 */

#include "vision.h"
#include "util/console.h"
#include "wifi.h"
#include "errors.h"
#include "util/sys_time.h"
#include <string.h>

#include "protobuf-c/messages_robocup_ssl_wrapper.pb-c.h"

Vision vision;

static void* allocMem(void *allocator_data, size_t size)
{
	(void)allocator_data;

	if(vision.allocUsed + size > VISION_ALLOC_BUF_SIZE)
	{
		ConsolePrint("WARN: vision out of memory\r\n");
		return 0;
	}

	void* pMem = vision.pAllocBuf + vision.allocUsed;

	vision.allocUsed += size;

	return pMem;
}

static void freeMem(void *allocator_data, void *pointer)
{
	(void)allocator_data;
	(void)pointer;
}

int16_t VisionInput(uint8_t* pData, uint16_t length)
{
	vision.allocUsed = 0;	// free vision buffer

	SSLWrapperPacket* pPacket = 0;
	pPacket = ssl__wrapper_packet__unpack(&vision.allocator, length, pData);
	if(pPacket == 0)
		return ERROR_VISION_PARSE;

	if(pPacket->detection == 0)
		return 0;

	float procDelay = (float)(pPacket->detection->t_sent - pPacket->detection->t_capture);	// [s]
	procDelay += 0.001;	// 1ms transport delay
	if(procDelay < 0)
		procDelay = 0;

	uint32_t procDelayUs = (uint32_t)(procDelay*1e6f);
	uint32_t tCapture = SysTimeUSec()-procDelayUs;

	// YELLOW run
	uint32_t numRobots = pPacket->detection->n_robots_yellow;
	SSLDetectionRobot** ppRobots = pPacket->detection->robots_yellow;
	for(uint32_t i = 0; i < numRobots; i++)
	{
		SSLDetectionRobot* pRobot = ppRobots[i];

		if(pRobot->has_robot_id == 0 || pRobot->has_orientation == 0)
			continue;

		WifiSendPose(pRobot->robot_id, pRobot->x, pRobot->y, pRobot->orientation, tCapture);
	}

	// BLUE run
	numRobots = pPacket->detection->n_robots_blue;
	ppRobots = pPacket->detection->robots_blue;
	for(uint32_t i = 0; i < numRobots; i++)
	{
		SSLDetectionRobot* pRobot = ppRobots[i];

		if(pRobot->has_robot_id == 0 || pRobot->has_orientation == 0)
			continue;

		WifiSendPose(12+pRobot->robot_id, pRobot->x, pRobot->y, pRobot->orientation, tCapture);
	}

	return 0;
}

void VisionInit()
{
	memset(&vision, 0, sizeof(Vision));

	vision.allocator.alloc = &allocMem;
	vision.allocator.free = &freeMem;
	vision.allocator.allocator_data = 0;
}
