/*
 * wifi.h
 *
 *  Created on: 07.07.2014
 *      Author: AndreR
 */

#ifndef WIFI_H_
#define WIFI_H_

#include "util/nrf24_ex.h"
#include "util/spi_sync.h"
#include "util/flash_fs.h"
#include "constants.h"
#include "commands.h"

#include "ch.h"

typedef void(*BotDataAvailCallback)(uint8_t);

typedef struct _WifiConfig
{
	uint8_t channel;		// = frequency
	uint8_t speed;
	uint8_t maxBots;
	uint8_t fixedRuntime;
	uint32_t timeout;	// in [ms]
} WifiConfig;

typedef struct _WifiRobot
{
	uint8_t txData[WIFI_TX_SIZE];
	uint8_t rxData[WIFI_RX_SIZE];

	NRF24Queue queue;

	uint8_t online;
	uint32_t lastResponseTime;

	int16_t lastVisionPos[3];
	uint32_t tVisionCapture;	// SysTime, [us]

	SystemMatchCtrl matchCtrl;
	uint32_t lastRSTTime;
} WifiRobot;

typedef struct _WifiGlobal
{
	NRF24Ex nrf24;
	Mutex nrf24Mutex;

	WifiRobot bots[NRF24_MAX_BOT_ID+1];

	uint32_t runtime;	// in [us]

	BinarySemaphore timerSem;

	SPISync spi3;

	WifiConfig cfg;
	FlashFile* pCfgFile;

	BotDataAvailCallback cbBotData;

	uint32_t lastCall;
}WifiGlobal;

extern WifiGlobal wifi;

void	WifiInit();
msg_t	WifiTask(void* params);
void	WifiSendPose(uint16_t botId, float x, float y, float orient, uint32_t tCapture);
void	WifiSetBotDataCallback(BotDataAvailCallback cb);

void	WifiSetChannel(uint8_t channel);
void	WifiSetSpeed(uint8_t speed);
void	WifiSetMaxBot(uint8_t maxBots);
void	WifiSetFixedRuntime(uint8_t enable);
void	WifiSetTimeout(uint32_t ms);
void	WifiSaveConfig();
void	WifiPrintConfig();

#endif /* WIFI_H_ */
