/*
 * kicker.h
 *
 *  Created on: 20.02.2014
 *      Author: AndreR
 */

#ifndef KICKER_H_
#define KICKER_H_

#include "ch.h"
#include "util/flash_fs.h"

#define SPI1_ADC_SAMPLES 8

#define KICKER_EVENT_QUEUE_SIZE 10

typedef struct _KickerInternalConfig
{
	float maxVoltage;
	uint32_t cooldown;
} KickerInternalConfig;

typedef struct _Kicker
{
	uint16_t txData[SPI1_ADC_SAMPLES*2];
	union
	{
		uint16_t rxData[SPI1_ADC_SAMPLES*2];
		uint32_t rxData32[SPI1_ADC_SAMPLES];
	};

	FlashFile* pConfigFile;
	KickerInternalConfig config;
	float hystVoltage;

	float vCap;
	float iChg;

	float chgOffset;

	uint8_t autoCharge;
	uint8_t state;

	uint8_t autoDischarge;

	float stress;

	uint32_t lastKickDone;

	uint8_t kickCounter;

	Mailbox eventQueue;
	msg_t eventQueueData[KICKER_EVENT_QUEUE_SIZE];
} Kicker;

extern Kicker kicker;

void KickerInit();
void KickerSetConfig(float v, uint32_t cooldown);
void KickerSetVoltageHysteresis(float vHyst);
void KickerEnableCharge(uint8_t enable);
void KickerEnableAutoRecharge(uint8_t enable);
void KickerFire1(float duration);	// [s]
void KickerFire2(float duration);	// [s]
void KickerAutoDischarge();
msg_t KickerTask(void* params);

#endif /* KICKER_H_ */
