/*
 * motor.h
 *
 *  Created on: 02.02.2014
 *      Author: AndreR
 */

#ifndef MOTOR_H_
#define MOTOR_H_

#include <stdint.h>

#include "stm32f30x.h"
#include "intercom_constants.h"
#include "util/flash_fs.h"
#include "util/ema_filter.h"
#include "util/pid.h"
#include "util/lag_element.h"
#include "ch.h"

#define IN_CCM __attribute__((section(".ccmtext")))
#define IN_CCM_VAR __attribute__((section(".ccmvar")))

#define DRIVE_ADC_SAMPLES 16
#define DRIVE_MEAS_SAMPLES 100
#define DRIVE_MOVE_THRESHOLD 100

#define MOTOR_KN 3760.0f			// [RPM/Volt]
#define MOTOR_KE (1.0f/MOTOR_KN)	// [V/RPM]
#define MOTOR_R 0.403f				// [Ohm]
#define MOTOR_KM 0.00254f			// [Nm/A]

// W3  W2  W1
//  5   5   5
#define CCER_A 0x005
#define CCER_B 0x050
#define CCER_C 0x500
#define CCER_AB (CCER_A | CCER_B)
#define CCER_BC (CCER_B | CCER_C)
#define CCER_CA (CCER_C | CCER_A)
#define CCER_ABC (CCER_A | CCER_B | CCER_C)

struct _Motor;

typedef struct _Motor
{
	// new stuff
	volatile int32_t pwmVal;

	volatile uint8_t halPos;
	volatile int32_t comDt;
	volatile float curVelocity;	// current velocity [rounds per minute]

	uint8_t enableAutoCommutation;
	PID pidSpeed;

	// ### Setup ###
	// ADC calibration values
	uint32_t adcCal[2];

	// ### Measured values ###
	// ADC DMA storage area
	union
	{
		uint16_t adc[2][DRIVE_ADC_SAMPLES];
		uint32_t adc32[2][DRIVE_ADC_SAMPLES/2];
	};

	EMAFilter avgCurrent;
	EMAFilter biasCurrent;

	LagElementPT1 tempPT1;
	float temp; // [K] over environment
	uint8_t overheated;
} Motor;

extern Motor motor;
extern Motor* pMotor;

void MotorInit();

void MotorSetVoltage(float vol);
void MotorSetSpeed(float rpm);
void MotorEnableSpeedPID(uint8_t enable);
void MotorEnableAutoCommutation(uint8_t enable);

void MotorSetPhaseVoltagesSVPWM(float Va, float Vb, float Vc);
void MotorSetAlphaBeta(float Valpha, float Vbeta);
void MotorSetVoltageAngle(float Vref, float angle);
void MotorEnablePhase(uint32_t ccer);

#endif /* MOTOR_H_ */
