/*
 * usart1.c
 *
 *  Created on: 26.01.2014
 *      Author: AndreR
 */

#include "usart1.h"

#include "../constants.h"
#include "util/init_hal.h"

#include "led.h"

USART usart1;

void USART1_IRQHandler()
{
	USARTIRQ(&usart1);
}

void TIM6_DAC_IRQHandler()
{
	TIM6->SR = 0;
	TIM6->CR1 &= ~TIM_CR1_CEN;

	USARTTimerIRQ(&usart1);
}

static void startTimeout(uint32_t microSeconds)
{
	TIM6->ARR = microSeconds;
	TIM6->CR1 |= TIM_CR1_CEN;
}

void USART1Init()
{
	GPIOInitData gpioInit;

	// TIM6 setup
	RCC->APB1ENR |= RCC_APB1ENR_TIM6EN;

	TIM6->CR1 = 0;
	TIM6->CR2 = 0;
	TIM6->PSC = 71;		// = 1MHz
	TIM6->ARR = 999;	// = 1ms
	TIM6->SR = 0;
	TIM6->DIER = TIM_DIER_UIE;

	NVICEnableIRQ(TIM6_DAC_IRQn, IRQL_USART1);

	// enable peripheral clock
	RCC->APB2ENR |= RCC_APB2ENR_USART1EN;

	// provide data storage
	static uint8_t usart2TxData[USART1_FIFO_SIZE];
	static uint8_t usart2RxData[USART1_FIFO_SIZE];

	// map data and configure
	USARTData data2;
	data2.pDataRx = usart2RxData;
	data2.rxSize = USART1_FIFO_SIZE;
	data2.pDataTx = usart2TxData;
	data2.txSize = USART1_FIFO_SIZE;
	data2.maxPacketSize = USART1_MAX_PACKET_SIZE;
	data2.pDMA = DMA1;
	data2.rxStream = 5;
	data2.txStream = 4;
	data2.rxIRQn = DMA1_Channel5_IRQn;
	data2.txIRQn = DMA1_Channel4_IRQn;
	data2.usartIRQn = USART1_IRQn;
	data2.IRQL = IRQL_USART1;
	data2.baudrate = 3000000;
	data2.pStartTimer = &startTimeout;

	// Init USART util layer
	USARTInit(&usart1, USART1, data2);

	// GPIO setup
	gpioInit.mode = GPIO_MODE_AF;
	gpioInit.alternate = 7;	// TIM6
	gpioInit.otype = GPIO_OTYPE_PUSH_PULL;
	gpioInit.ospeed = GPIO_OSPEED_F3_50MHZ;
	gpioInit.pupd = GPIO_PUPD_UP;
	GPIOInit(GPIOB, GPIO_PIN_6 | GPIO_PIN_7, &gpioInit);
}
