/*
 * supply.h
 *
 *  Created on: 28.01.2014
 *      Author: AndreR
 */

#ifndef SUPPLY_H_
#define SUPPLY_H_

#include "ch.h"
#include "stm32f30x.h"
#include "util/lag_element.h"

#define NUM_SUPPLY_SAMPLES 20
#define NUM_IR_SAMPLES 16

#define SUPPLY_LOWPASS_500_TAPS 19

typedef void(*IRDoneCallback)();

typedef struct _Supply
{
	uint16_t adcData[2*NUM_SUPPLY_SAMPLES];
	uint16_t irData[NUM_IR_SAMPLES];
	uint8_t irCur;
	uint16_t irResult[2];

	arm_fir_decimate_instance_f32 batFir;
	arm_fir_decimate_instance_f32 shuntFir;
	float32_t batFirData[SUPPLY_LOWPASS_500_TAPS+NUM_SUPPLY_SAMPLES-1];
	float32_t shuntFirData[SUPPLY_LOWPASS_500_TAPS+NUM_SUPPLY_SAMPLES-1];

	LagElementPT1 batLag;
	LagElementPT1 shuntLag;

	float bat;
	float shunt;

	uint8_t batEmpty;

	uint32_t adcCalSupply;
	uint32_t adcCalIr;

	IRDoneCallback irDoneCb;

#ifdef DEBUG
	DMA_TypeDef* pDMA;
	DMA_Channel_TypeDef* pDMACh;
	ADC_TypeDef* pADC;
#endif
} Supply;

extern Supply supply;

void SupplyInit();

#endif /* SUPPLY_H_ */
