/*
 * config_main.c
 *
 *  Created on: 17.05.2015
 *      Author: AndreR
 */

#include "config_main.h"

#include "util/intercom_reliable.h"
#include "util/log.h"
#include "network.h"

static void outputNetwork(void* pData, uint16_t dataLength)
{
	NetworkSendPacketReliable((PacketHeader*)pData, pData+PACKET_HEADER_SIZE, dataLength-PACKET_HEADER_SIZE);
}

void ConfigMainInit()
{
	ConfigInit(&outputNetwork);
}

void ConfigNetworkInput(PacketHeader* pHeader, uint8_t* pData, uint16_t dataLength)
{
	IntercomHeader iHeader;
	iHeader.section = INTERCOM_SECTION_CONFIG;

	if(pHeader->section != SECTION_CONFIG)
		return;

	switch(pHeader->cmd)
	{
		case CMD_CONFIG_QUERY_FILE_LIST:
		{
			iHeader.command = CMD_CONFIG_QUERY_FILE_LIST;
			iHeader.address = INTERCOM_ADDRESS_MEDIA | INTERCOM_ADDRESS_LEFT | INTERCOM_ADDRESS_KD;

			IntercomReliableSend(&iHeader, 0, 0);

			ConfigInternalInjectEvent(CONFIG_EVENT_SEND_FILE_LIST);
		}
		break;
		case CMD_CONFIG_ITEM_DESC:
		{
			ConfigItemDesc* pDesc = (ConfigItemDesc*)pData;

			uint16_t targetProc = (pDesc->id & 0xF000) >> 12;
			pDesc->id &= 0x0FFF;	// target processor extracted, remove processor bits

			switch(targetProc)
			{
				case INTERCOM_ID_MAIN:
				{
					ConfigInternalInjectEvent(CONFIG_EVENT_SEND_ITEM_DESC | pDesc->id | (((msg_t)pDesc->element) << 16));
				}
				break;
				case INTERCOM_ID_MEDIA:
				{
					iHeader.command = CMD_CONFIG_ITEM_DESC;
					iHeader.address = INTERCOM_ADDRESS_MEDIA;
					IntercomReliableSend(&iHeader, dataLength, pData);
				}
				break;
				case INTERCOM_ID_KD:
				{
					iHeader.command = CMD_CONFIG_ITEM_DESC;
					iHeader.address = INTERCOM_ADDRESS_KD;
					IntercomReliableSend(&iHeader, dataLength, pData);
				}
				break;
				case INTERCOM_ID_LEFT:
				case INTERCOM_ID_RIGHT:
				{
					iHeader.command = CMD_CONFIG_ITEM_DESC;
					iHeader.address = INTERCOM_ADDRESS_LEFT;
					IntercomReliableSend(&iHeader, dataLength, pData);
				}
				break;
				default:
					break;
			}
		}
		break;
		case CMD_CONFIG_WRITE:
		{
			ConfigReadWrite* pWrite = (ConfigReadWrite*)pData;

			if(dataLength < sizeof(ConfigReadWrite))
			{
				LogErrorC("Invalid datalength", dataLength);
				return;
			}

			uint16_t targetProc = (pWrite->id & 0xF000) >> 12;
			pWrite->id &= 0x0FFF;	// target processor extracted, remove processor bits

			switch(targetProc)
			{
				case INTERCOM_ID_MAIN:
				{
					uint16_t size = dataLength - sizeof(ConfigReadWrite);
					pData += sizeof(ConfigReadWrite);

					ConfigInternalWriteFile(pWrite->id, pData, size);
				}
				break;
				case INTERCOM_ID_MEDIA:
				{
					iHeader.command = CMD_CONFIG_WRITE;
					iHeader.address = INTERCOM_ADDRESS_MEDIA;
					IntercomReliableSend(&iHeader, dataLength, pData);
				}
				break;
				case INTERCOM_ID_KD:
				{
					iHeader.command = CMD_CONFIG_WRITE;
					iHeader.address = INTERCOM_ADDRESS_KD;
					IntercomReliableSend(&iHeader, dataLength, pData);
				}
				break;
				case INTERCOM_ID_LEFT:
				case INTERCOM_ID_RIGHT:
				{
					iHeader.command = CMD_CONFIG_WRITE;
					iHeader.address = INTERCOM_ADDRESS_LEFT;
					IntercomReliableSend(&iHeader, dataLength, pData);

					iHeader.address = INTERCOM_ADDRESS_RIGHT;
					IntercomReliableSend(&iHeader, dataLength, pData);
				}
				break;
				default:
					break;
			}

		}
		break;
		case CMD_CONFIG_READ:
		{
			ConfigReadWrite* pRead = (ConfigReadWrite*)pData;

			uint16_t targetProc = (pRead->id & 0xF000) >> 12;
			pRead->id &= 0x0FFF;	// target processor extracted, remove processor bits

			switch(targetProc)
			{
				case INTERCOM_ID_MAIN:
				{
					ConfigInternalInjectEvent(CONFIG_EVENT_SEND_FILE | pRead->id);
				}
				break;
				case INTERCOM_ID_MEDIA:
				{
					iHeader.command = CMD_CONFIG_READ;
					iHeader.address = INTERCOM_ADDRESS_MEDIA;
					IntercomReliableSend(&iHeader, dataLength, pData);
				}
				break;
				case INTERCOM_ID_KD:
				{
					iHeader.command = CMD_CONFIG_READ;
					iHeader.address = INTERCOM_ADDRESS_KD;
					IntercomReliableSend(&iHeader, dataLength, pData);
				}
				break;
				case INTERCOM_ID_LEFT:
				case INTERCOM_ID_RIGHT:
				{
					iHeader.command = CMD_CONFIG_READ;
					iHeader.address = INTERCOM_ADDRESS_LEFT;
					IntercomReliableSend(&iHeader, dataLength, pData);
				}
				break;
				default:
					break;
			}
		}
		break;
	}
}
