/*
 * ctrl_tigga.h
 *
 *  Created on: 26.02.2015
 *      Author: AndreR
 */

// TIGGA is pretty clever, he can control a robot :)

#ifndef CTRL_TIGGA_H_
#define CTRL_TIGGA_H_

#include <arm_math.h>
#include <stdint.h>
#include "util/config.h"
#include "util/ema_filter.h"
#include "util/kf.h"
#include "util/ukf.h"
#include "util/pid.h"
#include "util/trajectory.h"
#include "util/trajectory_orient.h"
#include "util/lag_element.h"

#include "../ctrl.h"

#define CTRL_TIGGA_VEL_STATE_BUFFER_SIZE 50

typedef struct _CtrlTiggaVelState
{
	uint32_t timestamp;	// [us], SysTime
	float vel[3];
} CtrlTiggaVelState;

typedef struct PACKED _CtrlTiggaConfigPos
{
	PIDParams pidParamsXY;
	PIDParams pidParamsW;
	uint8_t posCtrlInterval;
	float posPostRuntime;
	uint8_t accFeedforward;
	float trajTimeOffset;
	float lagVelPT1TimeConstant;
} CtrlTiggaConfigPos;

typedef struct PACKED _CtrlTiggaConfigFilter
{
	float statePos;
	float stateVel;

	float sensorVision;
	float sensorGyro;
	float sensorEncodersXYOffset; // Offset
	float sensorEncodersXYGain; // Gain
	float sensorEncodersW;
	float sensorAccelerometerOffset;
	float sensorAccelerometerGain;

	float visionDelay;

	float encMeasTrust;
} CtrlTiggaConfigFilter;

typedef struct _CtrlTigga
{
	// state estimation stuff
	KF kf[3];
	float kfData[3][KF_DATA_SIZE(2, 1, 3, 3)];

	float accBias[2];
	float accVel[2];
	LagElementPT1 accPT1[2];

	// control stuff
	float accFeedforward[3];
	PID pidPos[3];

	float outXYWForce[3];
	arm_matrix_instance_f32 matOutXYWForce; // 3x1

	float outMotorTorque[4];
	arm_matrix_instance_f32 matOutMotorTorque; // 4x1

	Trajectory2D trajXY;
	TrajectoryOrient trajW;
	float trajTime;

	uint8_t enableTraj;
	uint8_t posActive;

	CtrlTiggaVelState velStateBuffer[CTRL_TIGGA_VEL_STATE_BUFFER_SIZE];
	uint16_t velStateBufIndex;

	float setpointVel[3];
	float setpointPos[3];
	uint8_t newSetpointPos;
	float transitionTime;

	LagElementPT1 velLag[3];

	struct PACKED _enc
	{
		float xy2x[128];
		float xy2y[128];
		float xy2w[128];
		uint8_t saved;
	} enc;

	FlashFile* pEncFile;

	uint16_t encCover[128];

	EMAFilter avgTraj;
	EMAFilter avgSensorFusion;
	EMAFilter avgControl;

	CtrlTiggaConfigPos configPos;
	CtrlTiggaConfigFilter configFilter;
	ConfigFile* pConfigFilePos;
	ConfigFile* pConfigFileFilter;

	LogCtrlTiggaPIDs log;
} CtrlTigga;

extern CtrlTigga ctrlTigga;
extern CtrlInstance ctrlTiggaInstance;

void CtrlTiggaInitCfg();
void CtrlTiggaSetPIDPosParamsXY(float p, float i, float d, float max);
void CtrlTiggaSetPIDPosParamsW(float p, float i, float d, float max);
void CtrlTiggaSetVelTarget(float x, float y, float w);
void CtrlTiggaSetPosTarget(float x, float y, float w, float t);
void CtrlTiggaSetPosVelTarget(float* data);
float CtrlTiggaGetEncoderCoverage();

#endif /* CTRL_TIGGA_H_ */
