/*
 * usart1.c
 *
 *  Created on: 25.05.2011
 *      Author: AndreR
 */

#include "usart1.h"

#include "stm32f407xx.h"
#include "util/init_hal.h"
#include "util/bit_banding.h"

USART1Data usart1;

#define ClearTXEI() BBPeriphReset(USART1_BASE + 0x0C, 7)
#define SetTXEI() BBPeriphSet(USART1_BASE + 0x0C, 7)
#define GetTXEI() BBPeriphGet(USART1_BASE + 0x0C, 7)
#define ClearRXI() BBPeriphReset(USART1_BASE + 0x0C, 5)
#define SetRXI() BBPeriphSet(USART1_BASE + 0x0C, 5)
#define GetRXI() BBPeriphGet(USART1_BASE + 0x0C, 5)

void USART1_IRQHandler()
{
	CH_IRQ_PROLOGUE();

	uint32_t sr = USART1->SR;

	if(sr & USART_SR_TXE)	// enabled by TXEIE
	{
		// Transmitter empty, cleared by write to DR
		chSysLockFromIsr();
		uint32_t data = chOQGetI(&usart1.outQueue);
		chSysUnlockFromIsr();

		if(data == Q_EMPTY)
			ClearTXEI();
		else
			USART1->DR = data;
	}

	if(sr & USART_SR_RXNE)	// enabled by RXNEIE
	{
		// read data register not empty, cleared by: read DR
		uint8_t data = USART1->DR;

		chSysLockFromIsr();
		chIQPutI(&usart1.inQueue, data);
		chSysUnlockFromIsr();
	}

	CH_IRQ_EPILOGUE();
}

static void txReadyCb(OutputQueue* pOut)
{
	(void)pOut;

	SetTXEI();
}

void USART1Init()
{
	chIQInit(&usart1.inQueue, usart1.rxData, USART1_RX_SIZE, 0, 0);
	chOQInit(&usart1.outQueue, usart1.txData, USART1_TX_SIZE, &txReadyCb, 0);

	// init port
	GPIOInitData gpioInit;
	gpioInit.mode = GPIO_MODE_AF;
	gpioInit.otype = GPIO_OTYPE_PUSH_PULL;
	gpioInit.ospeed = GPIO_OSPEED_F4_2MHZ;
	gpioInit.alternate = 7;
	gpioInit.pupd = GPIO_PUPD_UP;
	GPIOInit(GPIOB, GPIO_PIN_6 | GPIO_PIN_7, &gpioInit);

	// USART1
	RCC->APB2ENR |= RCC_APB2ENR_USART1EN;

	USART1->CR1 = USART_CR1_TE | USART_CR1_RE;
	USART1->CR2 = 0;
	USART1->CR3 = 0;
	USART1->BRR = USARTCalcBRR(921600, USART1, 0);

	NVICEnableIRQ(USART1_IRQn, IRQL_USART1);

	USART1->CR1 |= USART_CR1_UE;

	SetRXI();
}
