/*
 * usart2.c
 *
 *  Created on: 03.11.2012
 *      Author: AndreR
 */

#include "usart2.h"

#include "util/init_hal.h"
#include "../constants.h"

USART usart2;

void USART2_IRQHandler()
{
	USARTIRQ(&usart2);
}

void TIM2_IRQHandler()
{
	TIM2->SR = 0;
	TIM2->CR1 &= ~TIM_CR1_CEN;

	USARTTimerIRQ(&usart2);
}

static void startTimeout(uint32_t microSeconds)
{
	TIM2->ARR = microSeconds;
	TIM2->CR1 |= TIM_CR1_CEN;
}

void USART2Init()
{
	// TIM2 setup
	RCC->APB1ENR |= RCC_APB1ENR_TIM2EN;

	TIM2->CR1 = 0;
	TIM2->PSC = 83; 	// clock is now 1MHz
	TIM2->ARR = 999;	// =1ms
	TIM2->SR = 0;
	TIM2->DIER = TIM_DIER_UIE;

	NVICEnableIRQ(TIM2_IRQn, IRQL_USART2);

	// enable peripheral clock
	RCC->APB1ENR |= RCC_APB1ENR_USART2EN;

	// provide data storage
	static uint8_t usart2TxData[USART2_FIFO_SIZE];
	static uint8_t usart2RxData[USART2_FIFO_SIZE];

	// map data and configure
	USARTData data;
	data.pDataRx = usart2RxData;
	data.rxSize = USART2_FIFO_SIZE;
	data.pDataTx = usart2TxData;
	data.txSize = USART2_FIFO_SIZE;
	data.maxPacketSize = USART2_MAX_PACKET_SIZE;
	data.pDMA = DMA1;
	data.dmaChannel = 4;
	data.rxStream = 5;
	data.txStream = 6;
	data.rxIRQn = DMA1_Stream5_IRQn;
	data.txIRQn = DMA1_Stream6_IRQn;
	data.usartIRQn = USART2_IRQn;
	data.IRQL = IRQL_USART2;
	data.baudrate = 3000000;
	data.pStartTimer = &startTimeout;

	// Init USART util layer
	USARTInit(&usart2, USART2, data);

	// GPIO setup
	GPIOInitData gpioInit;
	gpioInit.mode = GPIO_MODE_AF;
	gpioInit.alternate = 7; 	// USART2
	gpioInit.otype = GPIO_OTYPE_PUSH_PULL;
	gpioInit.ospeed = GPIO_OSPEED_F4_25MHZ;
	gpioInit.pupd = GPIO_PUPD_UP;
	GPIOInit(GPIOA, GPIO_PIN_2 | GPIO_PIN_3, &gpioInit);
}
