/*
 * usart6.c
 *
 *  Created on: 03.11.2012
 *      Author: AndreR
 */

#include "usart6.h"

#include "util/init_hal.h"
#include "../constants.h"

USART usart6;

void USART6_IRQHandler()
{
	USARTIRQ(&usart6);
}

void DMA2_Stream1_IRQHandler()
{
//	USARTDMARx(&usart6);
}

// USART2 TX
void DMA2_Stream6_IRQHandler()
{
//	USARTDMATx(&usart6);
}

void TIM1_TRG_COM_TIM11_IRQHandler()
{
	TIM11->SR = 0;
	TIM11->CR1 &= ~TIM_CR1_CEN;

	USARTTimerIRQ(&usart6);
}

static void startTimeout(uint32_t microSeconds)
{
	TIM11->ARR = microSeconds;
	TIM11->CR1 |= TIM_CR1_CEN;
}

void USART6Init()
{
	// TIM11 setup
	RCC->APB2ENR |= RCC_APB2ENR_TIM11EN;

	TIM11->CR1 = 0;
	TIM11->PSC = 83; 	// clock is now 1MHz
	TIM11->ARR = 999;	// =1ms
	TIM11->SR = 0;
	TIM11->DIER = TIM_DIER_UIE;

	NVICEnableIRQ(TIM1_TRG_COM_TIM11_IRQn, IRQL_USART2);

	// enable peripheral clock
	RCC->APB2ENR |= RCC_APB2ENR_USART6EN;

	// provide data storage
	static uint8_t usart6TxData[USART6_FIFO_SIZE];
	static uint8_t usart6RxData[USART6_FIFO_SIZE];

	// map data and configure
	USARTData data;
	data.pDataRx = usart6RxData;
	data.rxSize = USART6_FIFO_SIZE;
	data.pDataTx = usart6TxData;
	data.txSize = USART6_FIFO_SIZE;
	data.maxPacketSize = USART6_MAX_PACKET_SIZE;
	data.pDMA = DMA2;
	data.dmaChannel = 5;
	data.rxStream = 1;
	data.txStream = 6;
	data.rxIRQn = DMA2_Stream1_IRQn;
	data.txIRQn = DMA2_Stream6_IRQn;
	data.usartIRQn = USART6_IRQn;
	data.IRQL = IRQL_USART6;
	data.baudrate = 5250000;
	data.pStartTimer = &startTimeout;

	// Init USART util layer
	USARTInit(&usart6, USART6, data);

	// GPIO setup
	GPIOInitData gpioInit;
	gpioInit.mode = GPIO_MODE_AF;
	gpioInit.alternate = 8; 	// USART6
	gpioInit.otype = GPIO_OTYPE_PUSH_PULL;
	gpioInit.ospeed = GPIO_OSPEED_F4_25MHZ;
	gpioInit.pupd = GPIO_PUPD_UP;
	GPIOInit(GPIOC, GPIO_PIN_6 | GPIO_PIN_7, &gpioInit);
}
