/*
 * dect.h
 *
 *  Created on: 01.07.2010
 *      Author: AndreR
 */

#ifndef __NETWORK_H__
#define __NETWORK_H__

#include "util/nrf24_ex.h"
#include "util/flash_fs.h"
#include "util/fifo_lin.h"

#include "constants.h"
#include "commands.h"
#include "intercom_constants.h"

#include "ch.h"

typedef void(*PosCallback)();

#define NETWORK_REL_EVENT_SIZE 4

typedef struct _Network
{
	uint8_t txProcessingBuf[NETWORK_TX_PROCESSING_SIZE];
	uint8_t rxProcessingBuf[NETWORK_RX_PROCESSING_SIZE];

	uint8_t netPrint[NETWORK_PRINT_SIZE];

	uint8_t reliableBuf[NETWORK_RELIABLE_BUF_SIZE];
	FifoLin reliableCmds;
	uint16_t seq;
	msg_t relEventsData[NETWORK_REL_EVENT_SIZE];
	Mailbox relEvents;

	BinarySemaphore timerSem;
	Mutex sendPacketMutex;
	Mutex printMutex;

	NRF24Queue queue;
	NRF24Ex nrf24;

	uint32_t lastUSec;
	uint32_t avgRxPeriod;

	PosCallback posCb;

	int16_t lastVisionPos[3];
	uint32_t lastVisionPosTimestamp;
	uint32_t lastVisionDelay;

	ConfigNetwork config;
	FlashFile* pConfigFile;

	uint8_t botDisabled;
	uint32_t lastMatchCtrlTime;

	uint32_t lastUIWifiStatTime;

	uint8_t sumatraOnline;
	uint8_t bsOnline;

	uint8_t skillOverride;
} Network;

extern Network network;

void	NetworkInit();
msg_t	NetworkTask(void* params);
msg_t	NetworkReliableTask(void* params);
int16_t NetworkSendPacket(PacketHeader* pHeader, void* _pData, uint16_t dataLength);
void	NetworkSendPacketRaw(void* _pData, uint16_t dataLength);
int16_t NetworkSendPacketReliable(PacketHeader* pHeader, void* _pData, uint16_t dataLength);
void	NetworkSetPosCallback(PosCallback cb);
void	NetworkSaveConfig();
void	NetworkSendAnnouncement(uint32_t id);

#endif /* DECT_H_ */
