/*
 * skills.c
 *
 *  Created on: 24.06.2015
 *      Author: AndreR
 */

#include "skills.h"
#include "ctrl.h"
#include <string.h>

#include "skills/skill_basics.h"
#include "skills/skill_sine.h"
#include "skills/skill_enc_train.h"
#include "skills/skill_tune_pid.h"

Skills skills;

SkillInstanceId skillInstances[] = {
	{   1, &skillLocalVel },
	{   2, &skillGlobalVel },
	{   3, &skillGlobalPos },
	{   4, &skillSine },
	{   5, &skillEncTrain },
	{   6, &skillTunePid },
};

void SkillsInit()
{
	chMtxInit(&skills.input.mutex);

	skills.numSkills = sizeof(skillInstances)/sizeof(skillInstances[0]);
	skills.pInstanceId = &skillInstances[0];
}

// this is called from the network task whenever a new match cmd arrives
void SkillsSetInput(uint8_t skillId, const uint8_t* pData)
{
	chMtxLock(&skills.input.mutex);

	skills.input.skillId = skillId;
	memcpy(skills.input.data, pData, SYSTEM_MATCH_CTRL_USER_DATA_SIZE);

	chMtxUnlock();
}

// this is called in CTRL task after state estimation and before control
void SkillsExecute()
{
	chMtxLock(&skills.input.mutex);

	SkillInstanceId* pNewInstance = 0;

	// find new skill instance
	for(uint16_t i = 0; i < skills.numSkills; i++)
	{
		if(skillInstances[i].id == skills.input.skillId)
		{
			// found the new one
			pNewInstance = &skillInstances[i];
			break;
		}
	}

	// check if a skill change is required
	if(skills.pInstanceId != pNewInstance)
	{
		// call exit function if specified
		if(skills.pInstanceId && skills.pInstanceId->pInstance->pExit)
			(*skills.pInstanceId->pInstance->pExit)();

		// assign new skill
		skills.pInstanceId = pNewInstance;

		// call init function if specified
		if(skills.pInstanceId && skills.pInstanceId->pInstance->pInit)
			(*skills.pInstanceId->pInstance->pInit)();
	}

	if(skills.pInstanceId)
	{
		// activate motors if this skill is valid
		ctrl.motorsOn = 1;

		skills.output.updated = 0;
		if(skills.pInstanceId->pInstance->pSkill)
		{
			(*skills.pInstanceId->pInstance->pSkill)(skills.input.data, &skills.output);
		}
	}
	else
	{
		ctrl.motorsOn = 0;
	}

	chMtxUnlock();
}
