/*
 * skill_basics.c
 *
 *  Created on: 24.06.2015
 *      Author: AndreR
 */

#include "skill_basics.h"

#include "../ctrl.h"
#include "../hal/buzzer.h"
#include "../util/console.h"

static void localVelRun(void* pData, SkillOutput* pOutput);
static void globalVelRun(void* pData, SkillOutput* pOutput);
static void globalPosRun(void* pData, SkillOutput* pOutput);

SkillInstance skillLocalVel = { 0, &localVelRun, 0 };
SkillInstance skillGlobalVel = { 0, &globalVelRun, 0 };
SkillInstance skillGlobalPos = { 0, &globalPosRun, 0 };

typedef struct PACKED _XYWMapping
{
	int16_t x;
	int16_t y;
	int16_t w;
} XYWMapping;

typedef struct PACKED _XYWTMapping
{
	int16_t x;
	int16_t y;
	int16_t w;
	float t;
} XYWTMapping;

static void localVelRun(void* pData, SkillOutput* pOutput)
{
	XYWMapping* pVel = (XYWMapping*)pData;

	pOutput->xyw[0] = pVel->x*0.001f;
	pOutput->xyw[1] = pVel->y*0.001f;
	pOutput->xyw[2] = pVel->w*0.001f;

	pOutput->mode = SKILL_OUT_MODE_LOCAL_VEL;
	pOutput->updated = 1;
}

static void globalVelRun(void* pData, SkillOutput* pOutput)
{
	XYWMapping* pVel = (XYWMapping*)pData;

	pOutput->xyw[0] = pVel->x*0.001f;
	pOutput->xyw[1] = pVel->y*0.001f;
	pOutput->xyw[2] = pVel->w*0.001f;

	pOutput->mode = SKILL_OUT_MODE_GLOBAL_VEL;
	pOutput->updated = 1;
}

static void globalPosRun(void* pData, SkillOutput* pOutput)
{
	XYWTMapping* pPos = (XYWTMapping*)pData;

	pOutput->xyw[0] = pPos->x*0.001f;
	pOutput->xyw[1] = pPos->y*0.001f;
	pOutput->xyw[2] = pPos->w*0.001f;
	pOutput->xyw[3] = pPos->t;

	pOutput->mode = SKILL_OUT_MODE_GLOBAL_POS;
	pOutput->updated = 1;
}
