/*
 * skill_tune_pid.c
 *
 *  Created on: 26.06.2015
 *      Author: NicolaiO
 */

#include "skill_tune_pid.h"

#include "util/angle_math.h"
#include "arm_math.h"
#include "../ctrl.h"
#include "../ctrl/ctrl_tigga.h"
#include "../network.h"
#include "util/console.h"
#include "util/sys_time.h"

#define ANGLE_DIFF_THRESHOLD 0.05f
#define DIST_DIFF_THRESHOLD 0.01f

static void tunePidInit();
static void tunePidRun(void* pData, SkillOutput* pOutput);

SkillInstance skillTunePid = { &tunePidInit, &tunePidRun, 0 };

static struct _SkillTunePid
{
	int state;
	float setPos[3];
	float initPos[3];
	float tStill;
	float tStart;
} tunePid;

static void tunePidInit()
{
	tunePid.state = 0;
	tunePid.tStill = 0;
	tunePid.tStart = SysTime();
	memcpy(tunePid.initPos, ctrl.state.pos, sizeof(float) * 3);
}

static uint8_t positionReached()
{
	float angleDiff = AngleNormalize(tunePid.setPos[2]-ctrl.state.pos[2]);
	float xDiff = tunePid.setPos[0]-ctrl.state.pos[0];
	float yDiff = tunePid.setPos[1]-ctrl.state.pos[1];
	float xyDiff = sqrtf(xDiff*xDiff + yDiff*yDiff);

	if(fabsf(angleDiff) < ANGLE_DIFF_THRESHOLD && xyDiff < DIST_DIFF_THRESHOLD)
	{
		if(tunePid.tStill == 0)
			tunePid.tStill = SysTime();
		else if(SysTime() - tunePid.tStill > 0.5f)
			return 1;
	} else {
		tunePid.tStill = 0;
	}
	return 0;
}

static void newSetPoint(float x, float y, float w)
{
	tunePid.setPos[0] = x;
	tunePid.setPos[1] = y;
	tunePid.setPos[2] = w;
	tunePid.state++;
}

static void tunePidRun(void* pData, SkillOutput* pOutput)
{
	(void)pData;
//	SkillTunePidInput* pInput = (SkillTunePidInput*)pData;

	switch(tunePid.state)
	{
		case 0:
			newSetPoint(tunePid.initPos[0], tunePid.initPos[1], 0);
			break;
		case 1:
			if(positionReached()) tunePid.state++;
			break;
		case 2:
			newSetPoint(tunePid.initPos[0], tunePid.initPos[1], 1);
			break;
		case 3:
			if(positionReached()) tunePid.state++;
			break;
		case 4:
			newSetPoint(tunePid.initPos[0], tunePid.initPos[1], 3);
			break;
		case 5:
			if(positionReached()) tunePid.state++;
			break;
		case 6:
			newSetPoint(tunePid.initPos[0], tunePid.initPos[1], 0);
			break;
		case 7:
			if(positionReached()) tunePid.state++;
			break;
		case 8:
			newSetPoint(tunePid.initPos[0]+1.5, tunePid.initPos[1], 0);
			break;
		case 9:
			if(positionReached()) tunePid.state++;
			break;
		case 10:
			newSetPoint(tunePid.initPos[0]+1.5, tunePid.initPos[1], PI/2);
			break;
		case 11:
			if(positionReached()) tunePid.state++;
			break;
		case 12:
			newSetPoint(tunePid.initPos[0], tunePid.initPos[1], PI/2);
			break;
		case 13:
			if(positionReached()) tunePid.state++;
			break;
		default:
			tunePid.state = 0;
			break;
	}

	memcpy(pOutput->xyw, tunePid.setPos, sizeof(float) * 3);

	pOutput->mode = SKILL_OUT_MODE_GLOBAL_POS;
	pOutput->updated = 1;


	StatusFeedbackPid feedback;
	for(int i=0;i<3;i++)
	{
		feedback.t = (int16_t)((SysTime() - tunePid.tStart) * 1000.0f);
		feedback.in[i] =  (int16_t)(ctrlTigga.pidPos[i].lastMeasurement*1000.0f);
		feedback.out[i] = (int16_t)(ctrlTigga.pidPos[i].output*1000.0f);
		feedback.set[i] = (int16_t)(ctrlTigga.pidPos[i].setpoint*1000.0f);
	}

	PacketHeader header;
	header.section = SECTION_STATUS;
	header.cmd = CMD_STATUS_FEEDBACK_PID;
	NetworkSendPacket(&header, (uint8_t*) &feedback, sizeof(StatusFeedbackPid));
}
