/*
 * spi3.h
 *
 *  Created on: 19.01.2014
 *      Author: AndreR
 */

#ifndef SPI3_H_
#define SPI3_H_

#include "mot_exchange_t.h"
#include "stm32f407xx.h"
#include "ch.h"
#include <stdint.h>
#include "util/fifo_lin.h"

#define SPI3_RESERVE_SIZE	256

#define SPI3_FIFO_SIZE_TX	1024
#define SPI3_FIFO_SIZE_RX	1024

#define SPI3_SLAVE_LEFT		0
#define SPI3_SLAVE_RIGHT	1

typedef struct _SPI3Slave
{
	SPIMot2MainData rxData;
	SPIMain2MotData txData;
	GPIO_TypeDef* pCSPort;
	uint16_t csPin;

	uint8_t fifoDataTx[SPI3_FIFO_SIZE_TX];
	uint8_t fifoDataRx[SPI3_FIFO_SIZE_RX];

	FifoLin txFifo;
	FifoLin rxFifo;

	uint16_t id;
} SPI3Slave;

typedef void(*SPI3Callback)(void*);

typedef void(*SPI3AboutToSendCb)(SPI3Slave*, uint8_t slaveID);

typedef struct _SPI3Global
{
	SPI3Slave slave[2];

	Mailbox eventQueue;
	msg_t eventQueueData[4];

	SPI3Callback callback;
	void* pUserData;

	SPI3AboutToSendCb aboutToSendCb;

	BinarySemaphore transferSem;
	BinarySemaphore timerSem;

	uint32_t lastUSec;
	uint32_t avgTxPeriod;

	uint32_t lastSyncTime;

#ifdef DEBUG
	DMA_Stream_TypeDef* pTxDma;
	DMA_Stream_TypeDef* pRxDma;
	SPI_TypeDef* pSPI;
	DMA_TypeDef* pDMA;
#endif
} SPI3Global;

extern SPI3Global spi3;

void	SPI3Init();
msg_t	SPI3Task(void* params);

#endif /* SPI3_H_ */
