/*
 * ui.c
 *
 *  Created on: 22.05.2014
 *      Author: AndreR
 */

#include "ui.h"
#include "util/intercom.h"
#include "intercom_constants.h"
#include "network.h"

UI ui = { .cfgMisc = { 0, 0xFF } };

static void uiCallback(const IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	(void)pUserData;
	(void)dataLength;

	switch(pHeader->command)
	{
		case IC_UI_WIFI_BOTID:
		{
			UIWifiBotID* pID = (UIWifiBotID*)pData;

			NRF24ExChangeId(&network.nrf24, pID->id);
			network.config.botId = pID->id;
		}
		break;
		case IC_UI_WIFI_SETTINGS:
		{
			UIWifiSettings* pSet = (UIWifiSettings*)pData;

			NRF24ExChangeSpeed(&network.nrf24, pSet->speed);
			NRF24ExChangeChannel(&network.nrf24, pSet->channel);

			network.config.speed = pSet->speed;
			network.config.channel = pSet->channel;
		}
		break;
		case IC_UI_WIFI_SAVE:
		{
			NetworkSaveConfig();
		}
		break;
		case IC_UI_FEATURES:
		{
			uint16_t* pFeatures = (uint16_t*)pData;

			ui.cfgMisc.features = *pFeatures;

			FlashFSWrite(&ui.pBotStatusFile, &ui.cfgMisc, sizeof(ConfigMisc));
		}
		break;
	}
}

void UIInit()
{
	chMBInit(&ui.eventQueue, ui.eventQueueData, UI_EVENT_QUEUE_SIZE);

	FlashFSOpenOrCreate("botStatus", 0, &ui.cfgMisc, sizeof(ConfigMisc), &ui.pBotStatusFile);

	IntercomClient client;
	client.pUserData = 0;
	client.rxCallback = &uiCallback;
	client.section = INTERCOM_SECTION_UI;

	IntercomAddClient(client);
}

msg_t UITask(void* params)
{
	(void)params;
	msg_t event;

	chRegSetThreadName("UI");

	while(1)
	{
		chMBFetch(&ui.eventQueue, &event, MS2ST(500));

		IntercomHeader iHeader;
		iHeader.address = INTERCOM_ADDRESS_MEDIA;
		iHeader.section = INTERCOM_SECTION_PRESENTER;
		iHeader.command = IC_PRESENTER_CONFIG;

		IntercomSend(&iHeader, sizeof(ConfigNetwork), (uint8_t*)&network.config);

		iHeader.command = IC_PRESENTER_CFG_MISC;

		IntercomSend(&iHeader, sizeof(ConfigMisc), &ui.cfgMisc);
	}

	return 0;
}

void UISetHardwareID(uint8_t id)
{
	ui.cfgMisc.hardwareId = id;

	FlashFSWrite(&ui.pBotStatusFile, &ui.cfgMisc, sizeof(ConfigMisc));
}
