/*
 * desktop.c
 *
 *  Created on: 01.07.2014
 *      Author: AndreR
 */

#include "desktop.h"
#include "botcover.h"
#include "util/nrf24_ex.h"
#include "ids.h"
#include "util/intercom_reliable.h"
#include "util/console.h"
#include <stdio.h>
#include "version.h"

#include "DIALOG.h"
#include "TEXT.h"

static struct _DesktopHandles
{
	WM_HWIN desktop;

	WM_HWIN power;
	WM_HWIN bat;
	WM_HWIN cur;

	WM_HWIN wifi;
	WM_HWIN cover;
	WM_HWIN ch;
	WM_HWIN speed;

	WM_HWIN ir;

	WM_HWIN hwId;

	WM_HWIN kicker;
	WM_HWIN cap;
	WM_HWIN chg;
} handles;

static void cbKicker(WM_MESSAGE* pMsg)
{
	switch(pMsg->MsgId)
	{
		case WM_NOTIFY_PARENT:
		{
			if(pMsg->Data.v == WM_NOTIFICATION_RELEASED && WM_GetId(pMsg->hWinSrc) == GUI_ID_DESKTOP)
			{
				ConsolePrint("Discharge clicked!\r\n");

				IntercomHeader iHeader;
				iHeader.address = INTERCOM_ADDRESS_KD;
				iHeader.section = INTERCOM_SECTION_KD;
				iHeader.command = IC_KD_AUTOCHG;

				KickerChargeAuto autoChg = { 0 };

				IntercomReliableSend(&iHeader, sizeof(KickerChargeAuto), &autoChg);
			}
		}
		break;
		default:
			WM_DefaultProc(pMsg);
	}
}

static void cbWifi(WM_MESSAGE* pMsg)
{
	switch(pMsg->MsgId)
	{
		case WM_NOTIFY_PARENT:
		{
			if(pMsg->Data.v == WM_NOTIFICATION_RELEASED && WM_GetId(pMsg->hWinSrc) == GUI_ID_DESKTOP_BOTID)
			{
				ConsolePrint("BotID clicked!\r\n");

				WM_SendMessageNoPara(WM_HBKWIN, GUI_ID_DESKTOP_BOTID);
			}
		}
		break;
		default:
			WM_DefaultProc(pMsg);
	}
}


WM_HWIN DesktopInit()
{
	TEXT_SetDefaultFont(&GUI_Font8x12_ASCII);
	TEXT_SetDefaultTextColor(GUI_WHITE);

	handles.desktop = WINDOW_CreateEx(0, 34, 240, 241, WM_HBKWIN, WM_CF_SHOW, 0, 0, 0);
	WINDOW_SetBkColor(handles.desktop, GUI_BLACK);

	handles.power = WINDOW_CreateEx(0, 0, 240, 45, handles.desktop, WM_CF_SHOW, 0, 0, 0);
	WINDOW_SetBkColor(handles.power, GUI_BLACK);

	TEXT_CreateEx(5, 5, 70, 15, handles.power, WM_CF_SHOW, 0, 0, "Battery:");

	handles.bat = PROGBAR_CreateEx(5, 25, 110, 15, handles.power, WM_CF_SHOW, PROGBAR_CF_HORIZONTAL, 0);
	PROGBAR_SetBarColor(handles.bat, 0, 0x00AA00);
	PROGBAR_SetBarColor(handles.bat, 1, 0x444444);
	PROGBAR_SetFont(handles.bat, &GUI_Font8x12_ASCII);
	PROGBAR_SetTextColor(handles.bat, 0, 0xFFFFFF);
	PROGBAR_SetTextColor(handles.bat, 1, 0xFFFFFF);
	PROGBAR_SetMinMax(handles.bat, 10000, 12600);
	PROGBAR_SetValue(handles.bat, 11100);
	PROGBAR_SetText(handles.bat, "11.1V");

	TEXT_CreateEx(125, 5, 90, 15, handles.desktop, WM_CF_SHOW, 0, 0, "Usage:");

	handles.cur = TEXT_CreateEx(125, 25, 60, 15, handles.desktop, WM_CF_SHOW, 0, 0, 0);

	handles.wifi = WINDOW_CreateEx(0, 50, 240, 85, handles.desktop, WM_CF_SHOW, 0, 0, &cbWifi);
	WINDOW_SetBkColor(handles.wifi, GUI_BLACK);

	TEXT_CreateEx(5, 5, 70, 15, handles.wifi, WM_CF_SHOW, 0, 0, "Channel:");
	TEXT_CreateEx(85, 5, 70, 15, handles.wifi, WM_CF_SHOW, 0, 0, "Speed:");
	TEXT_CreateEx(165, 5, 70, 15, handles.wifi, WM_CF_SHOW, 0, 0, "ID:");

	handles.ch = TEXT_CreateEx(5, 25, 70, 15, handles.wifi, WM_CF_SHOW, 0, 0, "0");
	handles.speed = TEXT_CreateEx(85, 25, 70, 15, handles.wifi, WM_CF_SHOW, 0, 0, "2MBit/s");

	TEXT_CreateEx(5,  45, 70, 15, handles.wifi, WM_CF_SHOW, 0, 0, "Barrier:");
	handles.ir = TEXT_CreateEx(5, 65, 70, 15, handles.wifi, WM_CF_SHOW, 0, 0, "0/0");

	TEXT_CreateEx(85, 45, 70, 15, handles.wifi, WM_CF_SHOW, 0, 0, "HW ID:");
	handles.hwId = TEXT_CreateEx(85, 65, 70, 15, handles.wifi, WM_CF_SHOW, 0, 0, "255");

	handles.cover = BOTCOVER_CreateEx(165, 25, handles.wifi, WM_CF_SHOW, GUI_ID_DESKTOP_BOTID, 0);

	handles.kicker = WINDOW_CreateEx(0, 140, 240, 100, handles.desktop, WM_CF_SHOW, 0, 0, &cbKicker);
	WINDOW_SetBkColor(handles.kicker, GUI_BLACK);

	TEXT_CreateEx(5, 5, 110, 15, handles.kicker, WM_CF_SHOW, 0, 0, "Kicker Level:");
	TEXT_CreateEx(160, 5, 75, 15, handles.kicker, WM_CF_SHOW, 0, 0, "Chg Amps:");

	handles.chg = TEXT_CreateEx(160, 25, 75, 15, handles.kicker, WM_CF_SHOW, 0, 0, "0.0A");

	handles.cap = PROGBAR_CreateEx(5, 25, 110, 15, handles.kicker, WM_CF_SHOW, PROGBAR_CF_HORIZONTAL, 0);
	PROGBAR_SetBarColor(handles.cap, 0, 0x0000AA);
	PROGBAR_SetBarColor(handles.cap, 1, 0x444444);
	PROGBAR_SetFont(handles.cap, &GUI_Font8x12_ASCII);
	PROGBAR_SetTextColor(handles.cap, 0, 0xFFFFFF);
	PROGBAR_SetTextColor(handles.cap, 1, 0xFFFFFF);
	PROGBAR_SetMinMax(handles.cap, 0, 180);
	PROGBAR_SetValue(handles.cap, 5);
	PROGBAR_SetText(handles.cap, "0.0V");

	WM_HWIN hDis = BUTTON_CreateEx(5, 45, 230, 30, handles.kicker, WM_CF_SHOW, 0, GUI_ID_DESKTOP);
	BUTTON_SetText(hDis, "Discharge");
	BUTTON_SetFont(hDis, &GUI_Font20B_ASCII);
	BUTTON_SetTextColor(hDis, BUTTON_CI_PRESSED, GUI_RED);
	BUTTON_SetTextColor(hDis, BUTTON_CI_UNPRESSED, GUI_RED);
	BUTTON_SetBkColor(hDis, BUTTON_CI_PRESSED, GUI_DARKRED);
	BUTTON_SetBkColor(hDis, BUTTON_CI_UNPRESSED, GUI_GRAY);

	char version[20];
	sprintf(version, "%s - %.7s", "v2.1.0", GIT_REF);

	TEXT_CreateEx(60, 80, 140, 15, handles.kicker, WM_CF_SHOW, 0, 0, version);

	return handles.desktop;
}

void DesktopKdUpdate(KdFeedback* pFb)
{
	static char buf[10];

	snprintf(buf, 10, "%5.2fV", pFb->power.bat);
	PROGBAR_SetText(handles.bat, buf);
	PROGBAR_SetValue(handles.bat, (int16_t)(pFb->power.bat*1000));

	snprintf(buf, 10, "%6.1fW", pFb->power.cur*pFb->power.bat);
	TEXT_SetText(handles.cur, buf);

	snprintf(buf, 10, "%5.1fV", pFb->kicker.cap);
	PROGBAR_SetText(handles.cap, buf);
	PROGBAR_SetValue(handles.cap, (int)pFb->kicker.cap);

	snprintf(buf, 10, "%5.2fA", pFb->kicker.chg);
	TEXT_SetText(handles.chg, buf);

	snprintf(buf, 10, "%4hu/%4hu", pFb->barrier.level[0], pFb->barrier.level[1]);

	TEXT_SetText(handles.ir, buf);
}

void DesktopWifiUpdate(ConfigNetwork* pCfg)
{
	static char buf[10];

	BOTCOVER_SetId(handles.cover, pCfg->botId);

	snprintf(buf, 10, "%3hu", (uint16_t)pCfg->channel);
	TEXT_SetText(handles.ch, buf);

	if(pCfg->speed == 2)
		TEXT_SetText(handles.speed, "2MBit/s");
	else if(pCfg->speed == 0)
		TEXT_SetText(handles.speed, "250kBit/s");
	else
		TEXT_SetText(handles.speed, "1MBit/s");
}

void DesktopHwIdUpdate(uint8_t id)
{
	static char buf[10];

	snprintf(buf, 10, "%3hu", (uint16_t)id);

	TEXT_SetText(handles.hwId, buf);
}
