/*
 * features.c
 *
 *  Created on: 16.07.2014
 *      Author: AndreR
 */

#include "ids.h"
#include "features.h"
#include "DIALOG.h"
#include "util/intercom_reliable.h"
#include "intercom_constants.h"

static const char* featureNames[] = {"Move", "Dribbler", "Straight Kick", "Chip Kick", "Barrier"};

#define ID_HIDE			GUI_ID_FEATURES + 0
#define ID_BTN_SAVE		GUI_ID_FEATURES + 1

static struct _FeaturesHandles
{
	FRAMEWIN_Handle frame;

	WM_HWIN features[SYSTEM_MATCH_FEEDBACK_NUM_FEATURES];
} handles;

static void cbFrame(WM_MESSAGE* pMsg)
{
	switch(pMsg->MsgId)
	{
	case WM_NOTIFY_PARENT:
	{
		if(pMsg->Data.v == WM_NOTIFICATION_RELEASED && WM_GetId(pMsg->hWinSrc) == ID_HIDE)
		{
			WM_HideWindow(handles.frame);
		}
	}
	break;
	default:
		FRAMEWIN_Callback(pMsg);
	}
}

static void cbClient(WM_MESSAGE* pMsg)
{
	switch(pMsg->MsgId)
	{
		case WM_NOTIFY_PARENT:
		{
			if(pMsg->Data.v == WM_NOTIFICATION_RELEASED)
			{
				switch(WM_GetId(pMsg->hWinSrc))
				{
					case ID_BTN_SAVE:
					{
						uint16_t features = 0;
						for(uint8_t i = 0; i < SYSTEM_MATCH_FEEDBACK_NUM_FEATURES; i++)
						{
							if(CHECKBOX_GetState(handles.features[i]) == 1)
								features |= (1 << i);
						}

						IntercomHeader iHeader;
						iHeader.address = INTERCOM_ADDRESS_MAIN;
						iHeader.section = INTERCOM_SECTION_UI;
						iHeader.command = IC_UI_FEATURES;

						IntercomReliableSend(&iHeader, sizeof(uint16_t), &features);

						WM_HideWindow(handles.frame);
					}
					break;
				}
			}
		}
		break;
		default:
			WM_DefaultProc(pMsg);
	}
}

WM_HWIN FeaturesInit()
{
	handles.frame = FRAMEWIN_CreateEx(0, 30, 240, 300, 0, 0, 0, 0, "Configure Features", &cbClient);
	FRAMEWIN_SetTextAlign(handles.frame, GUI_TA_LEFT | GUI_TA_VCENTER);
	WM_HWIN close = FRAMEWIN_AddButton(handles.frame, FRAMEWIN_BUTTON_RIGHT, 0, ID_HIDE);
	BUTTON_SetText(close, "X");
	BUTTON_SetFont(close, &GUI_Font16B_ASCII);
	WM_SetCallback(handles.frame, &cbFrame);

	WM_HWIN hClient = WM_GetFirstChild(handles.frame);

	TEXT_SetDefaultTextColor(GUI_BLACK);

	for(uint8_t i = 0; i < SYSTEM_MATCH_FEEDBACK_NUM_FEATURES; i++)
	{
		handles.features[i] = CHECKBOX_CreateEx(5, 10+38*i, 110, 20, hClient, WM_CF_SHOW, 0, 0);
		CHECKBOX_SetFont(handles.features[i], &GUI_Font16B_ASCII);
		CHECKBOX_SetTextAlign(handles.features[i], GUI_TA_LEFT | GUI_TA_TOP);
		CHECKBOX_SetText(handles.features[i], featureNames[i]);
	}

	WM_HWIN save = BUTTON_CreateEx(25, 200, 180, 30, hClient, WM_CF_SHOW, 0, ID_BTN_SAVE);
	BUTTON_SetFont(save, &GUI_Font16B_ASCII);
	BUTTON_SetText(save, "Save");

	return handles.frame;
}

void FeaturesSet(uint16_t features)
{
	if(WM_IsVisible(handles.frame))
		return;

	for(uint8_t i = 0; i < SYSTEM_MATCH_FEEDBACK_NUM_FEATURES; i++)
	{
		uint16_t check = (features >> i) & 0x0001;

		CHECKBOX_SetState(handles.features[i], check);
	}
}
