/*
 * navbar.c
 *
 *  Created on: 12.10.2014
 *      Author: AndreR
 */

#include <stdint.h>
#include "DIALOG.h"

#include "../icons/icons.h"

#include "navbar.h"

#define NAVBAR_MAX_TABS 6

static struct _Navbat
{
	WM_HWIN hNav;

	struct _Tab
	{
		const GUI_BITMAP* pIcon;
		WM_HWIN hDesktop;
	} tabs[NAVBAR_MAX_TABS];

	uint8_t tabsUsed;

	uint8_t selectedTab;
} navbar;

static void winCallback(WM_MESSAGE* pMsg)
{
	switch(pMsg->MsgId)
	{
	case WM_PAINT:
	{
		int startPos = 120-navbar.tabsUsed*20;

		for(uint8_t i = 0; i < navbar.tabsUsed; i++)
		{
			if(i == navbar.selectedTab)
			{
				GUI_SetColor(GUI_RED);
				GUI_DrawRect(startPos+2, 10, startPos+37, 46);
				GUI_DrawGradientV(startPos+3, 11, startPos+36, 45, GUI_BLACK, GUI_RED);
			}

			GUI_DrawBitmap(navbar.tabs[i].pIcon, startPos+4, 12);

			startPos += 40;
		}
	}
	break;
	default:
		WM_DefaultProc(pMsg);
	}
}

WM_HWIN NavbarInit()
{
	navbar.tabsUsed = 0;
	navbar.selectedTab = 0;

	navbar.hNav = WINDOW_CreateEx(0, 265, 240, 55, WM_HBKWIN, WM_CF_SHOW | WM_CF_HASTRANS, 0, 0, &winCallback);
	WINDOW_SetBkColor(navbar.hNav, GUI_BLACK);

	return navbar.hNav;
}

uint8_t NavbarAddTab(const GUI_BITMAP* pIcon, WM_HWIN desktop)
{
	if(navbar.tabsUsed >= NAVBAR_MAX_TABS)
		return 0;

	uint8_t id = navbar.tabsUsed++;

	navbar.tabs[id].pIcon = pIcon;
	navbar.tabs[id].hDesktop = desktop;

	return id;
}

void NavbarChangeTab(uint8_t inc)
{
	uint8_t newId = 0;

	if(inc)
		newId = (navbar.selectedTab + 1)%navbar.tabsUsed;
	else
	{
		if(navbar.selectedTab == 0)
			newId = navbar.tabsUsed-1;
		else
			newId = navbar.selectedTab-1;
	}

	NavbarSelectTab(newId);
}

void NavbarSelectTab(uint8_t id)
{
	WM_ShowWindow(navbar.tabs[id].hDesktop);
	WM_HideWindow(navbar.tabs[navbar.selectedTab].hDesktop);

	WM_InvalidateWindow(navbar.hNav);

	navbar.selectedTab = id;
}
