/*
 * select_botid.c
 *
 *  Created on: 03.07.2014
 *      Author: AndreR
 */

#include "select_botid.h"

#include "ids.h"
#include "DIALOG.h"
#include "botcover.h"
#include "util/intercom_reliable.h"
#include "intercom_constants.h"

#include "util/console.h"

#define ID_HIDE			GUI_ID_SELECT_BOTID + 0
#define ID_BTN_YELLOW	GUI_ID_SELECT_BOTID + 1
#define ID_BTN_BLUE		GUI_ID_SELECT_BOTID + 2
#define ID_BOT			GUI_ID_SELECT_BOTID + 3

static struct _SelectBotIDHandles
{
	FRAMEWIN_Handle frame;
	WM_HWIN yWin;
	WM_HWIN bWin;
} handles;

static void cbFrame(WM_MESSAGE* pMsg)
{
	switch(pMsg->MsgId)
	{
	case WM_NOTIFY_PARENT:
	{
		if(pMsg->Data.v == WM_NOTIFICATION_RELEASED && WM_GetId(pMsg->hWinSrc) == ID_HIDE)
		{
			WM_HideWindow(handles.frame);
		}
	}
	break;
	default:
		FRAMEWIN_Callback(pMsg);
	}
}

static void cbClient(WM_MESSAGE* pMsg)
{
//	ConsolePrint("msg: %d, data.v: %d\r\n", pMsg->MsgId, pMsg->Data.v);

	switch(pMsg->MsgId)
	{
		case WM_NOTIFY_PARENT:
		{
			if(pMsg->Data.v == WM_NOTIFICATION_RELEASED)
			{
				switch(WM_GetId(pMsg->hWinSrc))
				{
					case ID_BTN_YELLOW:
					{
						WM_HideWindow(handles.bWin);
						WM_ShowWindow(handles.yWin);
					}
					break;
					case ID_BTN_BLUE:
					{
						WM_HideWindow(handles.yWin);
						WM_ShowWindow(handles.bWin);
					}
					break;
				}
			}
		}
		break;
		default:
			WM_DefaultProc(pMsg);
	}
}

static void cbID(WM_MESSAGE* pMsg)
{
//	ConsolePrint("msg: %d, data.v: %d\r\n", pMsg->MsgId, pMsg->Data.v);

	switch(pMsg->MsgId)
	{
		case WM_NOTIFY_PARENT:
		{
			if(pMsg->Data.v == WM_NOTIFICATION_RELEASED)
			{
				int id = WM_GetId(pMsg->hWinSrc);
				id -= ID_BOT;
				if(id < 24)
				{
					IntercomHeader iHeader;
					iHeader.address = INTERCOM_ADDRESS_MAIN;
					iHeader.section = INTERCOM_SECTION_UI;
					iHeader.command = IC_UI_WIFI_BOTID;

					UIWifiBotID wifiBotId;
					wifiBotId.id = (uint8_t)id;

					IntercomReliableSend(&iHeader, sizeof(UIWifiBotID), (uint8_t*)&wifiBotId);

					iHeader.command = IC_UI_WIFI_SAVE;
					IntercomReliableSend(&iHeader, 0, 0);
				}
				WM_HideWindow(handles.frame);
			}
		}
		break;
		default:
			WM_DefaultProc(pMsg);
	}
}

WM_HWIN SelectBotIDInit()
{
	handles.frame = FRAMEWIN_CreateEx(0, 30, 240, 300, 0, 0, 0, 0, "Select Bot ID", &cbClient);
	FRAMEWIN_SetTextAlign(handles.frame, GUI_TA_LEFT | GUI_TA_VCENTER);
	WM_HWIN close = FRAMEWIN_AddButton(handles.frame, FRAMEWIN_BUTTON_RIGHT, 0, ID_HIDE);
	BUTTON_SetText(close, "X");
	BUTTON_SetFont(close, &GUI_Font16B_ASCII);
	WM_SetCallback(handles.frame, &cbFrame);

	WM_HWIN hClient = WM_GetFirstChild(handles.frame);

	WM_HWIN btnYellow = BUTTON_CreateEx(5, 5, 110, 30, hClient, WM_CF_SHOW, 0, ID_BTN_YELLOW);
	BUTTON_SetBkColor(btnYellow, 0, GUI_YELLOW);
	BUTTON_SetFont(btnYellow, &GUI_Font16B_ASCII);
	BUTTON_SetText(btnYellow, "Yellow");

	WM_HWIN btnBlue = BUTTON_CreateEx(122, 5, 110, 30, hClient, WM_CF_SHOW, 0, ID_BTN_BLUE);
	BUTTON_SetBkColor(btnBlue, 0, GUI_BLUE);
	BUTTON_SetFont(btnBlue, &GUI_Font16B_ASCII);
	BUTTON_SetText(btnBlue, "Blue");

	WM_HWIN yWin = WINDOW_CreateEx(5, 40, 230, 215, hClient, WM_CF_SHOW, 0, 0, &cbID);

	BOTCOVER_CreateEx(0, 0, yWin, WM_CF_SHOW, ID_BOT+0, 0);
	BOTCOVER_CreateEx(83, 0, yWin, WM_CF_SHOW, ID_BOT+1, 1);
	BOTCOVER_CreateEx(165, 0, yWin, WM_CF_SHOW, ID_BOT+2, 2);

	BOTCOVER_CreateEx(0, 55, yWin, WM_CF_SHOW, ID_BOT+3, 3);
	BOTCOVER_CreateEx(83, 55, yWin, WM_CF_SHOW, ID_BOT+4, 4);
	BOTCOVER_CreateEx(165, 55, yWin, WM_CF_SHOW, ID_BOT+5, 5);

	BOTCOVER_CreateEx(0, 110, yWin, WM_CF_SHOW, ID_BOT+6, 6);
	BOTCOVER_CreateEx(83, 110, yWin, WM_CF_SHOW, ID_BOT+7, 7);
	BOTCOVER_CreateEx(165, 110, yWin, WM_CF_SHOW, ID_BOT+8, 8);

	BOTCOVER_CreateEx(0, 165, yWin, WM_CF_SHOW, ID_BOT+9, 9);
	BOTCOVER_CreateEx(83, 165, yWin, WM_CF_SHOW, ID_BOT+10, 10);
	BOTCOVER_CreateEx(165, 165, yWin, WM_CF_SHOW, ID_BOT+11, 11);

	WM_HWIN bWin = WINDOW_CreateEx(5, 40, 230, 215, hClient, 0, 0, 0, &cbID);

	BOTCOVER_CreateEx(0, 0, bWin, WM_CF_SHOW, ID_BOT+12, 12);
	BOTCOVER_CreateEx(83, 0, bWin, WM_CF_SHOW, ID_BOT+13, 13);
	BOTCOVER_CreateEx(165, 0, bWin, WM_CF_SHOW, ID_BOT+14, 14);

	BOTCOVER_CreateEx(0, 55, bWin, WM_CF_SHOW, ID_BOT+15, 15);
	BOTCOVER_CreateEx(83, 55, bWin, WM_CF_SHOW, ID_BOT+16, 16);
	BOTCOVER_CreateEx(165, 55, bWin, WM_CF_SHOW, ID_BOT+17, 17);

	BOTCOVER_CreateEx(0, 110, bWin, WM_CF_SHOW, ID_BOT+18, 18);
	BOTCOVER_CreateEx(83, 110, bWin, WM_CF_SHOW, ID_BOT+19, 19);
	BOTCOVER_CreateEx(165, 110, bWin, WM_CF_SHOW, ID_BOT+20, 20);

	BOTCOVER_CreateEx(0, 165, bWin, WM_CF_SHOW, ID_BOT+21, 21);
	BOTCOVER_CreateEx(83, 165, bWin, WM_CF_SHOW, ID_BOT+22, 22);
	BOTCOVER_CreateEx(165, 165, bWin, WM_CF_SHOW, ID_BOT+23, 23);

	handles.yWin = yWin;
	handles.bWin = bWin;

	return handles.frame;
}
