/*
 * wifi_settings.c
 *
 *  Created on: 16.07.2014
 *      Author: AndreR
 */

#include "ids.h"
#include "wifi_settings.h"
#include "DIALOG.h"
#include "DROPDOWN.h"
#include "util/console.h"
#include "numpad.h"
#include "commands.h"
#include "util/nrf24_ex.h"
#include "util/intercom_reliable.h"
#include "intercom_constants.h"
#include <stdint.h>
#include <stdio.h>

#define ID_HIDE			GUI_ID_WIFI_SETTINGS + 0
#define ID_SAVE			GUI_ID_WIFI_SETTINGS + 1

static struct _WifiSettingsHandles
{
	FRAMEWIN_Handle frame;

	WM_HWIN speed;
	WM_HWIN channel;
} handles;

static void cbFrame(WM_MESSAGE* pMsg)
{
	switch(pMsg->MsgId)
	{
	case WM_NOTIFY_PARENT:
	{
		if(pMsg->Data.v == WM_NOTIFICATION_RELEASED && WM_GetId(pMsg->hWinSrc) == ID_HIDE)
		{
			WM_HideWindow(handles.frame);
			NumpadSetVisible(0);
		}
	}
	break;
	default:
		FRAMEWIN_Callback(pMsg);
	}
}

static void cbClient(WM_MESSAGE* pMsg)
{
	switch(pMsg->MsgId)
	{
		case WM_NOTIFY_PARENT:
		{
			if(pMsg->Data.v == WM_NOTIFICATION_RELEASED)
			{
				switch(WM_GetId(pMsg->hWinSrc))
				{
					case ID_SAVE:
					{
						char buf[10];
						uint16_t ch;

						IntercomHeader iHeader;
						iHeader.address = INTERCOM_ADDRESS_MAIN;
						iHeader.section = INTERCOM_SECTION_UI;
						iHeader.command = IC_UI_WIFI_SETTINGS;

						UIWifiSettings wifiSet;
						EDIT_GetText(handles.channel, buf, 10);
						sscanf(buf, "%hu", &ch);
						wifiSet.channel = ch;

						switch(DROPDOWN_GetSel(handles.speed))
						{
						case 2: wifiSet.speed = 0; break;
						case 1: wifiSet.speed = 1; break;
						default: wifiSet.speed = 2; break;
						}

						ConsolePrint("Ch: %hu, Sp: %hu\r\n", (uint16_t)wifiSet.channel, (uint16_t)wifiSet.speed);

						IntercomReliableSend(&iHeader, sizeof(UIWifiSettings), (uint8_t*)&wifiSet);

						iHeader.command = IC_UI_WIFI_SAVE;
						IntercomReliableSend(&iHeader, 0, 0);

						WM_HideWindow(handles.frame);
						NumpadSetVisible(0);
					}
					break;
				}
			}
		}
		break;
		default:
			WM_DefaultProc(pMsg);
	}
}

WM_HWIN WifiSettingsInit()
{
	handles.frame = FRAMEWIN_CreateEx(0, 30, 240, 246, 0, 0, 0, 0, "Wifi Settings", &cbClient);
	FRAMEWIN_SetTextAlign(handles.frame, GUI_TA_LEFT | GUI_TA_VCENTER);
	WM_HWIN close = FRAMEWIN_AddButton(handles.frame, FRAMEWIN_BUTTON_RIGHT, 0, ID_HIDE);
	BUTTON_SetText(close, "X");
	BUTTON_SetFont(close, &GUI_Font16B_ASCII);
	WM_SetCallback(handles.frame, &cbFrame);

	WM_HWIN hClient = WM_GetFirstChild(handles.frame);

	TEXT_SetDefaultTextColor(GUI_BLACK);

	DROPDOWN_SetDefaultFont(&GUI_Font16B_ASCII);

	TEXT_CreateEx(5, 10, 80, 20, hClient, WM_CF_SHOW, 0, 0, "Speed");

	handles.speed = DROPDOWN_CreateEx(95, 5, 110, 80, hClient, WM_CF_SHOW, 0, 0);
	DROPDOWN_AddString(handles.speed, "2MBit/s");
	DROPDOWN_AddString(handles.speed, "1MBit/s");
	DROPDOWN_AddString(handles.speed, "250kBit/s");
	DROPDOWN_SetSel(handles.speed, 0);
	DROPDOWN_SetItemSpacing(handles.speed, 8);
	DROPDOWN_SetTextHeight(handles.speed, 20);

	TEXT_CreateEx(5, 55, 80, 20, hClient, WM_CF_SHOW, 0, 0, "Channel");

	handles.channel = EDIT_CreateEx(95, 45, 110, 30, hClient, WM_CF_SHOW, 0, 0, 3);
	EDIT_SetFont(handles.channel, &GUI_Font8x12_ASCII);

	BUTTON_Handle hSave = BUTTON_CreateEx(35, 100, 170, 30, hClient, WM_CF_SHOW, 0, ID_SAVE);
	BUTTON_SetText(hSave, "Save");

	return handles.frame;
}

void WifiUpdateSettings(ConfigNetwork* pConfig)
{
	char buf[10];

	if(WM_IsVisible(handles.frame))
		return;

	snprintf(buf, 10, "%hu", (uint16_t)pConfig->channel);

	EDIT_SetText(handles.channel, buf);

	int32_t sel = 0;
	switch(pConfig->speed)
	{
	case 1: sel = 1; break;
	case 0: sel = 2; break;
	default: sel = 0; break;
	}

	DROPDOWN_SetSel(handles.speed, sel);
}
