/*
 * ov7725.c
 *
 *  Created on: 02.12.2012
 *      Author: AndreR
 */

#include "ov7725.h"

#include "sram.h"

#include "tft.h"

#include "util/console.h"
#include "util/init_hal.h"
#include "../constants.h"

OV7725 ov7725;

static const uint8_t qvgaReg[][2] =
{
	{0x12, 0x80},

	{0x12, 0x46},
//	{0x3d, 0x03},
//	{0x17, 0x22},
//	{0x18, 0xa4},
//	{0x19, 0x07},
//	{0x1a, 0xf0},
	{0x32, 0x00},
//	{0x29, 0x50},
//	{0x2c, 0x78},
//	{0x2a, 0x00},
	{0x11, 0x01},//00/01/03/07 for 60/30/15/7.5fps

//	{0x42, 0x7f},
//	{0x4d, 0x09},
//	{0x63, 0xe0},
//	{0x64, 0xff},
//	{0x65, 0x2f},
//	{0x66, 0x00},
//	{0x67, 0x48},
//
//	{0x13, 0xf0},
//	{0x0d, 0x41},
//	{0x0f, 0xc5},
//	{0x14, 0x11},
	{0x22, 0x7f},//ff/7f/3f/1f for 60/30/15/7.5fps
	{0x23, 0x03},//01/03/07/0f for 60/30/15/7.5fps
	{0x24, 0x50},
	{0x25, 0x30},
	{0x26, 0xa1},

	{0x6b, 0xaa},
	{0x13, 0xff},

	{0x90, 0x05},
	{0x91, 0x01},
	{0x92, 0x03},
	{0x93, 0x00},

	{0x94, 0x5f},//b0
	{0x95, 0x53},//9d
	{0x96, 0x11},//13
	{0x97, 0x1a},//16
	{0x98, 0x3d},//7b
	{0x99, 0x5a},//91
	{0x9a, 0x1e},//1e

	{0x9b, 0x00},
	{0x9c, 0x25},
	{0x9e, 0x81},
	{0xa6, 0x06},

	//modified saturation initialization value by pw 2008-03-04
	{0xa7, 0x65},
	{0xa8, 0x65},

	{0x7e, 0x0c},
	{0x7f, 0x16},
	{0x80, 0x2a},
	{0x81, 0x4e},
	{0x82, 0x61},
	{0x83, 0x6f},
	{0x84, 0x7b},
	{0x85, 0x86},
	{0x86, 0x8e},
	{0x87, 0x97},
	{0x88, 0xa4},
	{0x89, 0xaf},
	{0x8a, 0xc5},
	{0x8b, 0xd7},
	{0x8c, 0xe8},
	{0x8d, 0x20},
#if 0
	{0x34, 0x00},
	{0x33, 0x40},
	{0x22, 0x99},
	{0x23, 0x03},
	{0x4a, 0x10},
	{0x49, 0x10},
	{0x4b, 0x14},
	{0x4c, 0x17},
	{0x46, 0x05},
	{0x0e, 0x65},
#endif
	{0x69, 0x5d},
	{0x0c, 0x00},

	{0x33, 0x49},//0x66/0x99
	{0x34, 0x01},//0x66/0x99
	{0x0e, 0x65},

	{0x13, 0xff}, //Enable banding Filter

	//sharpness strength modified by pw 2008-03-05
	{0xac, 0xdf},
	{0x8f, 0x04},

	{0xFF, 0x00},
};

static EXT_SRAM uint8_t picBuf[307200]; // sufficient for an 640x480 image

void DMA2_Stream1_IRQHandler()
{
	CH_IRQ_PROLOGUE();

	DMA2->LIFCR = DMA_LIFCR_CTCIF1;

	if(ov7725.mode == OV7725_MODE_STREAM_TO_DISPLAY)
	{
		*TFT_CMD_REG = TFT_REG_WRITE_MEMORY_START;
	}

	if(ov7725.mode == OV7725_MODE_SNAPSHOT)
	{
		chSysLockFromIsr();
		chBSemSignalI(&ov7725.acqDoneSem);
		chSysUnlockFromIsr();
	}

	CH_IRQ_EPILOGUE();
}

static void regInit()
{
	for(uint8_t i = 0; 1; i++)
	{
		if(qvgaReg[i][0] == 0xFF) // stop?
			break;

		OV7725WriteReg(qvgaReg[i][0], qvgaReg[i][1]);
	}
}

void OV7725Init()
{
	GPIOInitData gpioInit;

	chBSemInit(&ov7725.acqDoneSem, TRUE);

	gpioInit.mode = GPIO_MODE_AF;
	gpioInit.alternate = 0x0D;	// DCMI
	gpioInit.otype = GPIO_OTYPE_PUSH_PULL;
	gpioInit.ospeed = GPIO_OSPEED_F4_100MHZ;
	gpioInit.pupd = GPIO_PUPD_UP;

	GPIOInit(GPIOA, GPIO_PIN_4 | GPIO_PIN_6 | GPIO_PIN_9, &gpioInit);	// HSYNC, PCLK, D0
	GPIOInit(GPIOB, GPIO_PIN_6 | GPIO_PIN_7, &gpioInit);				// D5, VSYNC
	GPIOInit(GPIOC, GPIO_PIN_7 | GPIO_PIN_8 | GPIO_PIN_9, &gpioInit);	// D1, D2, D3
	GPIOInit(GPIOE, GPIO_PIN_4 | GPIO_PIN_5 | GPIO_PIN_6, &gpioInit);	// D4, D6, D7

	gpioInit.mode = GPIO_MODE_OUTPUT;
	gpioInit.ospeed = GPIO_OSPEED_F4_2MHZ;
	GPIOInit(GPIOA, GPIO_PIN_5 | GPIO_PIN_7, &gpioInit);	// FSIN, PWDN

	GPIOA->BSRRH = GPIO_PIN_5 | GPIO_PIN_7;  // PWDN low = normal mode, FSIN low

	// Init control interface
	SCCBInit(&ov7725.sccb, GPIOB, GPIO_PIN_10, GPIO_PIN_11, 0x42); // not joking, 0x42 is really the OV7725 device address

	chThdSleep(MS2ST(100));

	regInit();

	//--- DCMI Configuration ---

	// Enable DCMI clock
	RCC->AHB2ENR |= RCC_AHB2ENR_DCMIEN;

	DMA2_Stream1->CR = 0;
	DMA2_Stream1->FCR = 0;

	OV7725SetMode(OV7725_MODE_SNAPSHOT);

	DMA2_Stream1->CR |= DMA_SxCR_TCIE;	// enable transfer complete interrupt

	// NVIC setup
	NVICEnableIRQ(DMA2_Stream1_IRQn, IRQL_CAM);
}

int16_t OV7725WriteReg(uint16_t addr, uint8_t data)
{
	return SCCBWriteReg(&ov7725.sccb, addr, data);
}

int16_t OV7725ReadReg(uint16_t addr, uint8_t* pData)
{
	return SCCBReadReg(&ov7725.sccb, addr, pData);
}


void OV7725SetMode(uint8_t mode)
{
	DCMI->CR &= ~DCMI_CR_CAPTURE;
	DMA2_Stream1->CR &= ~DMA_SxCR_EN;
	DCMI->CR &= ~DCMI_CR_ENABLE;

	ov7725.mode = mode;

	DMA2_Stream1->CR = DMA_CHSEL(1) | DMA_DIR_PERIPH2MEM;

	DMA2_Stream1->CR = DMA_CHSEL(1) | DMA_DIR_PERIPH2MEM |
			DMA_PSIZE_4BYTE | DMA_MSIZE_2BYTE | DMA_PL_HIGH | DMA_MBURST_SINGLE | DMA_PBURST_SINGLE;
	DMA2_Stream1->FCR = DMA_SxFCR_DMDIS | DMA_FTH_FULL;
	DMA2_Stream1->PAR = (uint32_t)&DCMI->DR;
	DMA2_Stream1->NDTR = 38400;
	DMA2_Stream1->M0AR = 0;

	DCMI->CR = DCMI_CR_VSPOL | DCMI_CR_PCKPOL;

	if(mode == OV7725_MODE_SNAPSHOT)
	{
		DCMI->CR |= DCMI_CR_CM;	// snapshot mode

		DMA2_Stream1->M0AR = (uint32_t)picBuf;
		DMA2_Stream1->CR |= DMA_SxCR_MINC;
	}

	if(mode == OV7725_MODE_STREAM_TO_DISPLAY)
	{
		DMA2_Stream1->M0AR = (uint32_t)TFT_DATA_REG;
		DMA2_Stream1->CR |= DMA_SxCR_CIRC;

		DMA2_Stream1->CR |= DMA_SxCR_EN;
		DCMI->CR |= DCMI_CR_ENABLE;

		*TFT_CMD_REG = TFT_REG_WRITE_MEMORY_START;

		DCMI->CR |= DCMI_CR_CAPTURE;
	}
}

int16_t OV7725AcquireImage()
{
	int16_t result = 0;

	if(ov7725.mode != OV7725_MODE_SNAPSHOT)
		return 2;

	chBSemReset(&ov7725.acqDoneSem, TRUE);

	DMA2_Stream1->NDTR = 38400;

	DMA2->LIFCR = (DMA_LIFCR_CFEIF1 | DMA_LIFCR_CHTIF1 | DMA_LIFCR_CTCIF1 | DMA_LIFCR_CTEIF1);

	DMA2_Stream1->CR |= DMA_SxCR_EN;
	DCMI->CR |= DCMI_CR_ENABLE | DCMI_CR_CAPTURE;

	if(chBSemWaitTimeout(&ov7725.acqDoneSem, MS2ST(5000)) != RDY_OK)
		result = 1;

	DMA2_Stream1->CR &= ~DMA_SxCR_EN;
	DCMI->CR &= ~DCMI_CR_ENABLE;

	ConsolePrint("Captured %d pixels\r\n", (38400-DMA2_Stream1->NDTR)*2);

	return result;
}

uint8_t* OV7725GetPicBuf()
{
	return picBuf;
}
