/*
 * usart2.c
 *
 *  Created on: 03.11.2012
 *      Author: AndreR
 */

#include "usart2.h"

#include "util/init_hal.h"
#include "../constants.h"

USART usart2;

void USART2_IRQHandler()
{
	USARTIRQ(&usart2);
}

void TIM1_TRG_COM_TIM11_IRQHandler()
{
	TIM11->SR = 0;
	TIM11->CR1 &= ~TIM_CR1_CEN;

	USARTTimerIRQ(&usart2);
}

static void startTimeout(uint32_t microSeconds)
{
	TIM11->ARR = microSeconds;
	TIM11->CR1 |= TIM_CR1_CEN;
}

void USART2Init()
{
	// TIM11 setup
	RCC->APB2ENR |= RCC_APB2ENR_TIM11EN;

	TIM11->CR1 = 0;
	TIM11->PSC = 83; 	// clock is now 1MHz
	TIM11->ARR = 999;	// =1ms
	TIM11->SR = 0;
	TIM11->DIER = TIM_DIER_UIE;

	NVICEnableIRQ(TIM1_TRG_COM_TIM11_IRQn, IRQL_USART2);

	// enable peripheral clock
	RCC->APB1ENR |= RCC_APB1ENR_USART2EN;

	// provide data storage
	static uint8_t usart2TxData[USART2_FIFO_SIZE];
	static uint8_t usart2RxData[USART2_FIFO_SIZE];

	// map data and configure
	USARTData data2;
	data2.pDataRx = usart2RxData;
	data2.rxSize = USART2_FIFO_SIZE;
	data2.pDataTx = usart2TxData;
	data2.txSize = USART2_FIFO_SIZE;
	data2.maxPacketSize = USART2_MAX_PACKET_SIZE;
	data2.pDMA = DMA1;
	data2.dmaChannel = 4;
	data2.rxStream = 5;
	data2.txStream = 6;
	data2.rxIRQn = DMA1_Stream5_IRQn;
	data2.txIRQn = DMA1_Stream6_IRQn;
	data2.usartIRQn = USART2_IRQn;
	data2.IRQL = IRQL_USART2;
	data2.baudrate = 5250000;
	data2.pStartTimer = &startTimeout;

	// Init USART util layer
	USARTInit(&usart2, USART2, data2);

	// GPIO setup
	GPIOInitData gpioInit;
	gpioInit.mode = GPIO_MODE_AF;
	gpioInit.alternate = 7; 	// USART2
	gpioInit.otype = GPIO_OTYPE_PUSH_PULL;
	gpioInit.ospeed = GPIO_OSPEED_F4_25MHZ;
	gpioInit.pupd = GPIO_PUPD_UP;
	GPIOInit(GPIOA, GPIO_PIN_2 | GPIO_PIN_3, &gpioInit);
}
