/*
 * log_file.h
 *
 *  Created on: 24.08.2014
 *      Author: AndreR
 */

#ifndef LOG_FILE_H_
#define LOG_FILE_H_

#include <stdint.h>
#include "ch.h"
#include "util/fifo_block.h"
#include "fatfs/ff.h"

// Internals
#define LOGFILE_EVENT_START 0
#define LOGFILE_EVENT_UPDATE 1
#define LOGFILE_EVENT_CLOSE 2

typedef struct _LogFileEvent
{
	uint8_t eventType;
	const char* pFilename;
	uint32_t fileSize;
	uint32_t bytesPerSecIn;
	uint32_t writeSpeed;	// B/s
} LogFileEvent;

typedef void(*LogFileCb)(LogFileEvent*);

#define BUFFER_SIZE		(16384*2)
#define BUFFER_CHUNKS	4
#define LOG_FILE_QUEUE_SIZE	BUFFER_CHUNKS

typedef struct _LogFile
{
	FIL file;
	Mutex fileMutex;
	uint8_t ready;

	FifoBlock writeFifo;

	msg_t eventQueueData[LOG_FILE_QUEUE_SIZE];
	Mailbox eventQueue;

	char logFilename[64];

	LogFileEvent event;
	LogFileCb eventCb;
	uint32_t lastEventSize;

	uint32_t bytesPushed;	// short term
	uint32_t lastPushedTime;
} LogFile;

extern LogFile logFile;

void	LogFileInit();
int16_t LogFileOpen(const char* pName);
void	LogFileClose();
msg_t	LogFileTask(void* params);
void	LogFileWrite(void* pLogMsg);

#endif /* LOG_FILE_H_ */
