/*
 * prog_source_usb.c
 *
 *  Created on: 02.04.2015
 *      Author: AndreR
 */

#include "prog_source_usb.h"
#include "util/crc.h"
#include "util/console.h"

#include <string.h>

static void dataFunc(uint8_t procId, uint32_t offset, uint32_t size);
static void crcFunc(uint8_t procId, uint32_t startAddr, uint32_t endAddr);
static void sizeFunc(uint8_t procId);
static void open();
static void close();

ProgSourceUSB progSourceUSB = {.usb = {&dataFunc, &sizeFunc, &crcFunc, &open, &close, 100, PROG_SOURCE_USB_BUF}};

static const char* filenames[PROG_STORE_NUM_PROCESSORS] = {"main.bin", "media.bin", "left.bin", "right.bin", "kd.bin"};

void ProgSourceUSBInit()
{
	ProgStoreAddSource(&progSourceUSB.usb);
}

static void open()
{
	for(uint8_t procId = 0; procId < PROG_STORE_NUM_PROCESSORS; procId++)
	{
		f_open(&progSourceUSB.files[procId], filenames[procId], FA_READ | FA_OPEN_EXISTING);
	}
}

static void close()
{
	for(uint8_t procId = 0; procId < PROG_STORE_NUM_PROCESSORS; procId++)
	{
		f_close(&progSourceUSB.files[procId]);
	}
}

static void dataFunc(uint8_t procId, uint32_t offset, uint32_t size)
{
	memset(progSourceUSB.buf, 0xFF, PROG_SOURCE_USB_BUF);

	UINT bytesRead;
	FRESULT fresult;
	fresult = f_lseek(&progSourceUSB.files[procId], offset);
	if(fresult)
	{
		ProgStoreProvideError(procId, PROG_STORE_ERROR_NO_MEDIA);
		return;
	}

	fresult = f_read(&progSourceUSB.files[procId], progSourceUSB.buf, PROG_SOURCE_USB_BUF, &bytesRead);
	if(fresult)
	{
		ProgStoreProvideError(procId, PROG_STORE_ERROR_NO_MEDIA);
		return;
	}

	ProgStoreProvideData(procId, offset, progSourceUSB.buf, size);
}

static void sizeFunc(uint8_t procId)
{
	FRESULT fresult;
	FILINFO info;

	fresult = f_stat(filenames[procId], &info);
	if(fresult)
	{
		ProgStoreProvideError(procId, PROG_STORE_ERROR_NO_MEDIA);
		return;
	}

	ProgStoreProvideSize(procId, info.fsize);
}

static void crcFunc(uint8_t procId, uint32_t startAddr, uint32_t endAddr)
{
	UINT bytesRead;
	FRESULT fresult;
	fresult = f_lseek(&progSourceUSB.files[procId], startAddr);
	if(fresult)
	{
		ProgStoreProvideError(procId, PROG_STORE_ERROR_NO_MEDIA);
		return;
	}

	uint32_t addr = 0;
	uint32_t size = endAddr - startAddr;

	uint32_t crc = CRC32StartChecksumAlt();

	while(addr < size)
	{
		uint32_t readSize = PROG_SOURCE_USB_BUF;

		if(addr + PROG_SOURCE_USB_BUF > size)
			readSize = size-addr;

		memset(progSourceUSB.buf, 0xFF, PROG_SOURCE_USB_BUF);

		fresult = f_read(&progSourceUSB.files[procId], progSourceUSB.buf, readSize, &bytesRead);
		if(fresult)
		{
			ProgStoreProvideError(procId, PROG_STORE_ERROR_NO_MEDIA);
			break;
		}

		crc = CRC32FeedChecksumAlt(crc, progSourceUSB.buf, readSize);

		addr += readSize;
	}

	crc = CRC32StopChecksumAlt(crc);

	ProgStoreProvideCRC32(procId, startAddr, endAddr, crc);
}
