/*
 * prog_source_wifi.c
 *
 *  Created on: 02.04.2015
 *      Author: AndreR
 */

#include "prog_source_wifi.h"
#include "util/crc.h"
#include "util/console.h"
#include "commands.h"
#include "intercom_constants.h"
#include "util/intercom.h"
#include "presenter.h"
#include "util/log.h"

#include <string.h>

static void dataFunc(uint8_t procId, uint32_t offset, uint32_t size);
static void crcFunc(uint8_t procId, uint32_t startAddr, uint32_t endAddr);
static void sizeFunc(uint8_t procId);
static void open();
static void close();

ProgSourceWifi progSourceWifi = {.wifi = {&dataFunc, &sizeFunc, &crcFunc, &open, &close, 40, 56}};

static void networkCb(const IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	(void)pUserData;
	(void)pHeader;

	if(pHeader->command)
	{
		ProgStoreStart(PROG_STORE_SOURCE_WIFI);
		return;
	}

	PacketHeader* pPckHeader = (PacketHeader*)pData;
	pData += sizeof(PacketHeader);
	dataLength -= sizeof(PacketHeader);

	if(pPckHeader->section != SECTION_BOOTLOADER)
		return;

	switch(pPckHeader->cmd)
	{
		case CMD_BOOTLOADER_SIZE:
		{
			BootloaderSize* pSize = (BootloaderSize*)pData;

			if(pSize->size == 0xFFFFFFFF)
				ProgStoreProvideError(pSize->procId, PROG_STORE_ERROR_NO_MEDIA);
			else
				ProgStoreProvideSize(pSize->procId, pSize->size);
		}
		break;
		case CMD_BOOTLOADER_CRC32:
		{
			BootloaderCRC32* pCRC = (BootloaderCRC32*)pData;

			ProgStoreProvideCRC32(pCRC->procId, pCRC->startAddr, pCRC->endAddr, pCRC->crc);
		}
		break;
		case CMD_BOOTLOADER_DATA:
		{
			BootloaderData* pBData = (BootloaderData*)pData;
			pData += sizeof(BootloaderData);
			dataLength -= sizeof(BootloaderData);

			ProgStoreProvideData(pBData->procId, pBData->offset, pData, dataLength);
		}
		break;
	}
}

void ProgSourceWifiInit()
{
	IntercomClient iClient;
	iClient.pUserData = 0;
	iClient.rxCallback = &networkCb;
	iClient.section = INTERCOM_SECTION_NETWORK;
	IntercomAddClient(iClient);

	ProgStoreAddSource(&progSourceWifi.wifi);
}

static void open()
{
}

static void close()
{
}

static void dataFunc(uint8_t procId, uint32_t offset, uint32_t size)
{
	if(!presenter.lastWifiStat.sumatraOnline)
	{
		ProgStoreProvideError(procId, PROG_STORE_ERROR_NO_MEDIA);
		return;
	}

	uint8_t data[sizeof(PacketHeader)+sizeof(BootloaderRequestData)];
	PacketHeader* pPckHeader = (PacketHeader*)data;
	BootloaderRequestData* pReq = (BootloaderRequestData*)(data+sizeof(PacketHeader));

	pPckHeader->section = SECTION_BOOTLOADER;
	pPckHeader->cmd = CMD_BOOTLOADER_REQUEST_DATA;

	pReq->procId = procId;
	pReq->offset = offset;
	pReq->size = size;

	IntercomHeader iHeader;
	iHeader.address = INTERCOM_ADDRESS_MAIN;
	iHeader.section = INTERCOM_SECTION_NETWORK;
	iHeader.command = 0;

	IntercomSend(&iHeader, sizeof(data), data);
}

static void sizeFunc(uint8_t procId)
{
	if(!presenter.lastWifiStat.sumatraOnline)
	{
		ProgStoreProvideError(procId, PROG_STORE_ERROR_NO_MEDIA);
		return;
	}

	uint8_t data[sizeof(PacketHeader)+sizeof(BootloaderRequestSize)];
	PacketHeader* pPckHeader = (PacketHeader*)data;
	BootloaderRequestSize* pReq = (BootloaderRequestSize*)(data+sizeof(PacketHeader));

	pPckHeader->section = SECTION_BOOTLOADER;
	pPckHeader->cmd = CMD_BOOTLOADER_REQUEST_SIZE;

	pReq->procId = procId;

	IntercomHeader iHeader;
	iHeader.address = INTERCOM_ADDRESS_MAIN;
	iHeader.section = INTERCOM_SECTION_NETWORK;
	iHeader.command = 0;

	IntercomSend(&iHeader, sizeof(data), data);
}

static void crcFunc(uint8_t procId, uint32_t startAddr, uint32_t endAddr)
{
	if(!presenter.lastWifiStat.sumatraOnline)
	{
		ProgStoreProvideError(procId, PROG_STORE_ERROR_NO_MEDIA);
		return;
	}

	uint8_t data[sizeof(PacketHeader)+sizeof(BootloaderRequestCRC32)];
	PacketHeader* pPckHeader = (PacketHeader*)data;
	BootloaderRequestCRC32* pReq = (BootloaderRequestCRC32*)(data+sizeof(PacketHeader));

	pPckHeader->section = SECTION_BOOTLOADER;
	pPckHeader->cmd = CMD_BOOTLOADER_REQUEST_CRC32;

	pReq->procId = procId;
	pReq->startAddr = startAddr;
	pReq->endAddr = endAddr;

	IntercomHeader iHeader;
	iHeader.address = INTERCOM_ADDRESS_MAIN;
	iHeader.section = INTERCOM_SECTION_NETWORK;
	iHeader.command = 0;

	IntercomSend(&iHeader, sizeof(data), data);
}
