/*
 * prog_store.h
 *
 *  Created on: 01.04.2015
 *      Author: AndreR
 */

#ifndef PROG_STORE_H_
#define PROG_STORE_H_

#include "ch.h"
#include <stdint.h>

#define PROG_STORE_MAX_SOURCES 2
#define PROG_STORE_NUM_EVENTS 10
#define PROG_STORE_NUM_PROCESSORS 5

#define PROG_STORE_ERROR_TIMEOUT	1
#define PROG_STORE_ERROR_NO_MEDIA	2

#define PROG_STORE_SOURCE_USB		0
#define PROG_STORE_SOURCE_WIFI		1

typedef void(*ProgSourceRequestDataFunc)(uint8_t procId, uint32_t offset, uint32_t size);
typedef void(*ProgSourceRequestSizeFunc)(uint8_t procId);
typedef void(*ProgSourceRequestCRC32)(uint8_t procId, uint32_t startAddr, uint32_t endAddr);
typedef void(*ProgSourceOpenClose)();

typedef struct _ProgSource
{
	ProgSourceRequestDataFunc pDataFunc;
	ProgSourceRequestSizeFunc pSizeFunc;
	ProgSourceRequestCRC32 pCRC32Func;
	ProgSourceOpenClose pOpenFunc;
	ProgSourceOpenClose pCloseFunc;

	uint16_t timeout; // [ms]
	uint16_t maxTransferSize; // [B]
} ProgSource;

typedef struct _ProgStoreEvent
{
	uint8_t procId;
	uint16_t type;

	BinarySemaphore lockSem;

	union
	{
		int16_t errorId;

		struct _data
		{
			uint32_t offset;
			uint32_t size;
		} data;

		uint32_t size;

		struct _crc32
		{
			uint32_t startAddr;
			uint32_t endAddr;
			uint32_t crc;
		} crc32;
	};
} ProgStoreEvent;

typedef struct _ProgStore
{
	ProgSource sources[PROG_STORE_MAX_SOURCES];
	uint8_t usedSources;
	uint8_t useSource;

	ProgStoreEvent events[PROG_STORE_NUM_EVENTS];

	Thread* pTask;

	Mailbox eventQueue;
	msg_t eventQueueData[PROG_STORE_NUM_EVENTS];

	uint8_t* pProgs[PROG_STORE_NUM_PROCESSORS];

	VirtualTimer timeoutTimer;

	uint16_t progSeq;
	uint16_t expectedSeq;
	volatile uint8_t progAckd;
	BinarySemaphore progSem;
} ProgStore;

extern ProgStore progStore;

void ProgStoreInit();
void ProgStoreStart(uint8_t source);
void ProgStoreFlash();
int16_t ProgStoreAddSource(const ProgSource* pSource);

// ProgSource interface
void ProgStoreProvideError(uint8_t procId, int16_t error);
void ProgStoreProvideData(uint8_t procId, uint32_t offset, uint8_t* pData, uint32_t size);
void ProgStoreProvideSize(uint8_t procId, uint32_t size);
void ProgStoreProvideCRC32(uint8_t procId, uint32_t startAddr, uint32_t endAddr, uint32_t crc32);

#endif /* PROG_STORE_H_ */
