/*
    ChibiOS/RT - Copyright (C) 2006,2007,2008,2009,2010,
                 2011,2012,2013 Giovanni Di Sirio.

    This file is part of ChibiOS/RT.

    ChibiOS/RT is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    ChibiOS/RT is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

                                      ---

    A special exception to the GPL can be applied should you wish to distribute
    a combined work that includes ChibiOS/RT, without being obliged to provide
    the source code for any proprietary components. See the file exception.txt
    for full details of how and when the exception can be applied.
*/

/**
 * @file    GCC/ARMCMx/STM32F4xx/vectors.c
 * @brief   Interrupt vectors for the STM32F4xx family.
 *
 * @defgroup ARMCMx_STM32F4xx_VECTORS STM32F4xx Interrupt Vectors
 * @ingroup ARMCMx_SPECIFIC
 * @details Interrupt vectors for the STM32F4xx family.
 * @{
 */

#include "ch.h"

/**
 * @brief   Type of an IRQ vector.
 */
typedef void  (*irq_vector_t)(void);

/**
 * @brief   Type of a structure representing the whole vectors table.
 */
typedef struct {
  uint32_t      *init_stack;
  irq_vector_t  reset_vector;
  irq_vector_t  nmi_vector;
  irq_vector_t  hardfault_vector;
  irq_vector_t  memmanage_vector;
  irq_vector_t  busfault_vector;
  irq_vector_t  usagefault_vector;
  irq_vector_t  vector1c;
  irq_vector_t  vector20;
  irq_vector_t  vector24;
  irq_vector_t  vector28;
  irq_vector_t  svcall_vector;
  irq_vector_t  debugmonitor_vector;
  irq_vector_t  vector34;
  irq_vector_t  pendsv_vector;
  irq_vector_t  systick_vector;
  irq_vector_t  vectors[82];
} vectors_t;

#ifdef STM32F4XX
#define Vector40   WWDG_IRQHandler
#define Vector44   PVD_IRQHandler
#define Vector48   TAMP_STAMP_IRQHandler
#define Vector4C   RTC_WKUP_IRQHandler
#define Vector50   FLASH_IRQHandler
#define Vector54   RCC_IRQHandler
#define Vector58   EXTI0_IRQHandler
#define Vector5C   EXTI1_IRQHandler
#define Vector60   EXTI2_IRQHandler
#define Vector64   EXTI3_IRQHandler
#define Vector68   EXTI4_IRQHandler
#define Vector6C   DMA1_Stream0_IRQHandler
#define Vector70   DMA1_Stream1_IRQHandler
#define Vector74   DMA1_Stream2_IRQHandler
#define Vector78   DMA1_Stream3_IRQHandler
#define Vector7C   DMA1_Stream4_IRQHandler
#define Vector80   DMA1_Stream5_IRQHandler
#define Vector84   DMA1_Stream6_IRQHandler
#define Vector88   ADC_IRQHandler
#define Vector8C   CAN1_TX_IRQHandler
#define Vector90   CAN1_RX0_IRQHandler
#define Vector94   CAN1_RX1_IRQHandler
#define Vector98   CAN1_SCE_IRQHandler
#define Vector9C   EXTI9_5_IRQHandler
#define VectorA0   TIM1_BRK_TIM9_IRQHandler
#define VectorA4   TIM1_UP_TIM10_IRQHandler
#define VectorA8   TIM1_TRG_COM_TIM11_IRQHandler
#define VectorAC   TIM1_CC_IRQHandler
#define VectorB0   TIM2_IRQHandler
#define VectorB4   TIM3_IRQHandler
#define VectorB8   TIM4_IRQHandler
#define VectorBC   I2C1_EV_IRQHandler
#define VectorC0   I2C1_ER_IRQHandler
#define VectorC4   I2C2_EV_IRQHandler
#define VectorC8   I2C2_ER_IRQHandler
#define VectorCC   SPI1_IRQHandler
#define VectorD0   SPI2_IRQHandler
#define VectorD4   USART1_IRQHandler
#define VectorD8   USART2_IRQHandler
#define VectorDC   USART3_IRQHandler
#define VectorE0   EXTI15_10_IRQHandler
#define VectorE4   RTC_Alarm_IRQHandler
#define VectorE8   OTG_FS_WKUP_IRQHandler
#define VectorEC   TIM8_BRK_TIM12_IRQHandler
#define VectorF0   TIM8_UP_TIM13_IRQHandler
#define VectorF4   TIM8_TRG_COM_TIM14_IRQHandler
#define VectorF8   TIM8_CC_IRQHandler
#define VectorFC   DMA1_Stream7_IRQHandler
#define Vector100  FSMC_IRQHandler
#define Vector104  SDIO_IRQHandler
#define Vector108  TIM5_IRQHandler
#define Vector10C  SPI3_IRQHandler
#define Vector110  UART4_IRQHandler
#define Vector114  UART5_IRQHandler
#define Vector118  TIM6_DAC_IRQHandler
#define Vector11C  TIM7_IRQHandler
#define Vector120  DMA2_Stream0_IRQHandler
#define Vector124  DMA2_Stream1_IRQHandler
#define Vector128  DMA2_Stream2_IRQHandler
#define Vector12C  DMA2_Stream3_IRQHandler
#define Vector130  DMA2_Stream4_IRQHandler
#define Vector134  ETH_IRQHandler
#define Vector138  ETH_WKUP_IRQHandler
#define Vector13C  CAN2_TX_IRQHandler
#define Vector140  CAN2_RX0_IRQHandler
#define Vector144  CAN2_RX1_IRQHandler
#define Vector148  CAN2_SCE_IRQHandler
#define Vector14C  OTG_FS_IRQHandler
#define Vector150  DMA2_Stream5_IRQHandler
#define Vector154  DMA2_Stream6_IRQHandler
#define Vector158  DMA2_Stream7_IRQHandler
#define Vector15C  USART6_IRQHandler
#define Vector160  I2C3_EV_IRQHandler
#define Vector164  I2C3_ER_IRQHandler
#define Vector168  OTG_HS_EP1_OUT_IRQHandler
#define Vector16C  OTG_HS_EP1_IN_IRQHandler
#define Vector170  OTG_HS_WKUP_IRQHandler
#define Vector174  OTG_HS_IRQHandler
#define Vector178  DCMI_IRQHandler
#define Vector17C  CRYP_IRQHandler
#define Vector180  HASH_RNG_IRQHandler
#define Vector184  FPU_IRQHandler
#else
#define Vector40   WWDG_IRQHandler
#define Vector44   PVD_IRQHandler
#define Vector48   TAMPER_STAMP_IRQHandler
#define Vector4C   RTC_WKUP_IRQHandler
#define Vector50   FLASH_IRQHandler
#define Vector54   RCC_IRQHandler
#define Vector58   EXTI0_IRQHandler
#define Vector5C   EXTI1_IRQHandler
#define Vector60   EXTI2_TS_IRQHandler
#define Vector64   EXTI3_IRQHandler
#define Vector68   EXTI4_IRQHandler
#define Vector6C   DMA1_Channel1_IRQHandler
#define Vector70   DMA1_Channel2_IRQHandler
#define Vector74   DMA1_Channel3_IRQHandler
#define Vector78   DMA1_Channel4_IRQHandler
#define Vector7C   DMA1_Channel5_IRQHandler
#define Vector80   DMA1_Channel6_IRQHandler
#define Vector84   DMA1_Channel7_IRQHandler
#define Vector88   ADC1_2_IRQHandler
#define Vector8C   USB_HP_CAN1_TX_IRQHandler
#define Vector90   USB_LP_CAN1_RX0_IRQHandler
#define Vector94   CAN1_RX1_IRQHandler
#define Vector98   CAN1_SCE_IRQHandler
#define Vector9C   EXTI9_5_IRQHandler
#define VectorA0   TIM1_BRK_TIM15_IRQHandler
#define VectorA4   TIM1_UP_TIM16_IRQHandler
#define VectorA8   TIM1_TRG_COM_TIM17_IRQHandler
#define VectorAC   TIM1_CC_IRQHandler
#define VectorB0   TIM2_IRQHandler
#define VectorB4   TIM3_IRQHandler
#define VectorB8   TIM4_IRQHandler
#define VectorBC   I2C1_EV_IRQHandler
#define VectorC0   I2C1_ER_IRQHandler
#define VectorC4   I2C2_EV_IRQHandler
#define VectorC8   I2C2_ER_IRQHandler
#define VectorCC   SPI1_IRQHandler
#define VectorD0   SPI2_IRQHandler
#define VectorD4   USART1_IRQHandler
#define VectorD8   USART2_IRQHandler
#define VectorDC   USART3_IRQHandler
#define VectorE0   EXTI15_10_IRQHandler
#define VectorE4   RTC_Alarm_IRQHandler
#define VectorE8   USBWakeUp_IRQHandler
#define VectorEC   TIM8_BRK_IRQHandler
#define VectorF0   TIM8_UP_IRQHandler
#define VectorF4   TIM8_TRG_COM_IRQHandler
#define VectorF8   TIM8_CC_IRQHandler
#define VectorFC   ADC3_IRQHandler
#define Vector10C  SPI3_IRQHandler
#define Vector110  UART4_IRQHandler
#define Vector114  UART5_IRQHandler
#define Vector118  TIM6_DAC_IRQHandler
#define Vector11C  TIM7_IRQHandler
#define Vector120  DMA2_Channel1_IRQHandler
#define Vector124  DMA2_Channel2_IRQHandler
#define Vector128  DMA2_Channel3_IRQHandler
#define Vector12C  DMA2_Channel4_IRQHandler
#define Vector130  DMA2_Channel5_IRQHandler
#define Vector134  ADC4_IRQHandler
#define Vector140  COMP1_2_3_IRQHandler
#define Vector144  COMP4_5_6_IRQHandler
#define Vector148  COMP7_IRQHandler
#define Vector168  USB_HP_IRQHandler
#define Vector16C  USB_LP_IRQHandler
#define Vector170  USBWakeUp_RMP_IRQHandler
#define Vector184  FPU_IRQHandler
#endif

#if !defined(__DOXYGEN__)
extern uint32_t __main_stack_end__;
extern void ResetHandler(void);
extern void NMIVector(void);
extern void HardFaultVector(void);
extern void MemManageVector(void);
extern void BusFaultVector(void);
extern void UsageFaultVector(void);
extern void Vector1C(void);
extern void Vector20(void);
extern void Vector24(void);
extern void Vector28(void);
extern void SVCallVector(void);
extern void DebugMonitorVector(void);
extern void Vector34(void);
extern void PendSVVector(void);
extern void SysTickVector(void);
extern void Vector40(void);
extern void Vector44(void);
extern void Vector48(void);
extern void Vector4C(void);
extern void Vector50(void);
extern void Vector54(void);
extern void Vector58(void);
extern void Vector5C(void);
extern void Vector60(void);
extern void Vector64(void);
extern void Vector68(void);
extern void Vector6C(void);
extern void Vector70(void);
extern void Vector74(void);
extern void Vector78(void);
extern void Vector7C(void);
extern void Vector80(void);
extern void Vector84(void);
extern void Vector88(void);
extern void Vector8C(void);
extern void Vector90(void);
extern void Vector94(void);
extern void Vector98(void);
extern void Vector9C(void);
extern void VectorA0(void);
extern void VectorA4(void);
extern void VectorA8(void);
extern void VectorAC(void);
extern void VectorB0(void);
extern void VectorB4(void);
extern void VectorB8(void);
extern void VectorBC(void);
extern void VectorC0(void);
extern void VectorC4(void);
extern void VectorC8(void);
extern void VectorCC(void);
extern void VectorD0(void);
extern void VectorD4(void);
extern void VectorD8(void);
extern void VectorDC(void);
extern void VectorE0(void);
extern void VectorE4(void);
extern void VectorE8(void);
extern void VectorEC(void);
extern void VectorF0(void);
extern void VectorF4(void);
extern void VectorF8(void);
extern void VectorFC(void);
extern void Vector100(void);
extern void Vector104(void);
extern void Vector108(void);
extern void Vector10C(void);
extern void Vector110(void);
extern void Vector114(void);
extern void Vector118(void);
extern void Vector11C(void);
extern void Vector120(void);
extern void Vector124(void);
extern void Vector128(void);
extern void Vector12C(void);
extern void Vector130(void);
extern void Vector134(void);
extern void Vector138(void);
extern void Vector13C(void);
extern void Vector140(void);
extern void Vector144(void);
extern void Vector148(void);
extern void Vector14C(void);
extern void Vector150(void);
extern void Vector154(void);
extern void Vector158(void);
extern void Vector15C(void);
extern void Vector160(void);
extern void Vector164(void);
extern void Vector168(void);
extern void Vector16C(void);
extern void Vector170(void);
extern void Vector174(void);
extern void Vector178(void);
extern void Vector17C(void);
extern void Vector180(void);
extern void Vector184(void);
#endif

/**
 * @brief   STM32 vectors table.
 */
#if !defined(__DOXYGEN__)
__attribute__ ((used, section("vectors")))
#endif
vectors_t _vectors = {
  &__main_stack_end__,ResetHandler,       NMIVector,          HardFaultVector,
  MemManageVector,    BusFaultVector,     UsageFaultVector,   Vector1C,
  Vector20,           Vector24,           Vector28,           SVCallVector,
  DebugMonitorVector, Vector34,           PendSVVector,       SysTickVector,
  {
    Vector40,           Vector44,           Vector48,           Vector4C,
    Vector50,           Vector54,           Vector58,           Vector5C,
    Vector60,           Vector64,           Vector68,           Vector6C,
    Vector70,           Vector74,           Vector78,           Vector7C,
    Vector80,           Vector84,           Vector88,           Vector8C,
    Vector90,           Vector94,           Vector98,           Vector9C,
    VectorA0,           VectorA4,           VectorA8,           VectorAC,
    VectorB0,           VectorB4,           VectorB8,           VectorBC,
    VectorC0,           VectorC4,           VectorC8,           VectorCC,
    VectorD0,           VectorD4,           VectorD8,           VectorDC,
    VectorE0,           VectorE4,           VectorE8,           VectorEC,
    VectorF0,           VectorF4,           VectorF8,           VectorFC,
    Vector100,          Vector104,          Vector108,          Vector10C,
    Vector110,          Vector114,          Vector118,          Vector11C,
    Vector120,          Vector124,          Vector128,          Vector12C,
    Vector130,          Vector134,          Vector138,          Vector13C,
    Vector140,          Vector144,          Vector148,          Vector14C,
    Vector150,          Vector154,          Vector158,          Vector15C,
    Vector160,          Vector164,          Vector168,          Vector16C,
    Vector170,          Vector174,          Vector178,          Vector17C,
    Vector180,          Vector184
  }
};

/**
 * @brief   Unhandled exceptions handler.
 * @details Any undefined exception vector points to this function by default.
 *          This function simply stops the system into an infinite loop.
 *
 * @notapi
 */
#if !defined(__DOXYGEN__)
__attribute__ ((naked))
#endif
void _unhandled_exception(void) {

  while (TRUE)
    ;
}

void NMIVector(void) __attribute__((weak, alias("_unhandled_exception")));
void HardFaultVector(void) __attribute__((weak, alias("_unhandled_exception")));
void MemManageVector(void) __attribute__((weak, alias("_unhandled_exception")));
void BusFaultVector(void) __attribute__((weak, alias("_unhandled_exception")));
void UsageFaultVector(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector1C(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector20(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector24(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector28(void) __attribute__((weak, alias("_unhandled_exception")));
void SVCallVector(void) __attribute__((weak, alias("_unhandled_exception")));
void DebugMonitorVector(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector34(void) __attribute__((weak, alias("_unhandled_exception")));
void PendSVVector(void) __attribute__((weak, alias("_unhandled_exception")));
void SysTickVector(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector40(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector44(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector48(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector4C(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector50(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector54(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector58(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector5C(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector60(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector64(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector68(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector6C(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector70(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector74(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector78(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector7C(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector80(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector84(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector88(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector8C(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector90(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector94(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector98(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector9C(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorA0(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorA4(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorA8(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorAC(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorB0(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorB4(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorB8(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorBC(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorC0(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorC4(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorC8(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorCC(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorD0(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorD4(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorD8(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorDC(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorE0(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorE4(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorE8(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorEC(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorF0(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorF4(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorF8(void) __attribute__((weak, alias("_unhandled_exception")));
void VectorFC(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector100(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector104(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector108(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector10C(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector110(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector114(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector118(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector11C(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector120(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector124(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector128(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector12C(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector130(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector134(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector138(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector13C(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector140(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector144(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector148(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector14C(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector150(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector154(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector158(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector15C(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector160(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector164(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector168(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector16C(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector170(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector174(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector178(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector17C(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector180(void) __attribute__((weak, alias("_unhandled_exception")));
void Vector184(void) __attribute__((weak, alias("_unhandled_exception")));

/** @} */
