/*
 * commands.h
 *
 *  Created on: 24.05.2010
 *      Author: AndreR
 */

#ifndef __COMMANDS_H__
#define __COMMANDS_H__

#include <stdint.h>
#include "util/float16_t.h"

#ifndef PACKED
#define PACKED __attribute__ ((packed))
#endif

#define PACKET_HEADER_SIZE 2
typedef union PACKED _PacketHeader
{
	struct
	{
		uint8_t cmd;
		uint8_t section;
	};

	uint8_t _data[PACKET_HEADER_SIZE];
} PacketHeader;

#define PACKET_HEADER_EX_SIZE 4
typedef union PACKED _PacketHeaderEx
{
	struct
	{
		uint8_t cmd;
		uint8_t section;
		uint16_t seq;
	};

	uint8_t _data[PACKET_HEADER_EX_SIZE];
} PacketHeaderEx;

// SECTIONS
#define SECTION_SYSTEM			0
#define SECTION_DECT			1	// obsolete
#define SECTION_MOTOR			2
#define SECTION_KICKER			4
#define SECTION_MOVEMENT		5	// obsolete
#define SECTION_MULTICAST		6	// obsolete
#define SECTION_CTRL			7
#define SECTION_BASE_STATION	8
#define SECTION_BOOTLOADER		9
#define SECTION_SKILL			10	// obsolete
#define SECTION_CONFIG			11
#define SECTION_STATUS			12

#define SECTION_EXTENDED_PACKET	0x80

// SYSTEM
#define CMD_SYSTEM_POWER_LOG			2
#define CMD_SYSTEM_STATUS_MOVEMENT		3
#define CMD_SYSTEM_SET_IDENTITY			4
#define CMD_SYSTEM_ANNOUNCEMENT			5
#define CMD_SYSTEM_PING					6
#define CMD_SYSTEM_PONG					7
#define CMD_SYSTEM_BIG_PING				8
#define CMD_SYSTEM_SET_LOGS				9
#define CMD_SYSTEM_CONSOLE_PRINT		10
#define CMD_SYSTEM_CONSOLE_COMMAND		11
#define CMD_SYSTEM_FEATURES				12	// there is no struct for this command, it just consists of a list of [feature, state] pairs
#define CMD_SYSTEM_MATCH_CTRL			13
#define CMD_SYSTEM_MATCH_FEEDBACK		14
#define CMD_SYSTEM_PERFORMANCE			15
#define CMD_SYSTEM_QUERY				16
#define CMD_SYSTEM_STATUS_V2			0x80
#define CMD_SYSTEM_STATUS_EXT			0x81
#define CMD_SYSTEM_ACK					0xF0

// KICKER
#define CMD_KICKER_KICKV1				1
#define CMD_KICKER_STATUS				2
#define CMD_KICKER_SET_AUTOLOAD			3
#define CMD_KICKER_SET_MAX_CAP_LEVEL	4
#define CMD_KICKER_CHARGE				5
#define CMD_KICKER_SET_DUTY_CYCLE		6
#define CMD_KICKER_KICKV2				7
#define CMD_KICKER_STATUSV2				8
#define CMD_KICKER_CHARGE_MANUAL		9
#define CMD_KICKER_CHARGE_AUTO			10
#define CMD_KICKER_IR_LOG				11
#define CMD_KICKER_STATUSV3				12
#define CMD_KICKER_CONFIG				13

// MOTOR
#define CMD_MOTOR_MOVE					0
#define CMD_MOTOR_DRIBBLE				1
#define CMD_MOTOR_SET_PID_PARAMS		2
#define CMD_MOTOR_SET_MODE				3
#define CMD_MOTOR_SET_MANUAL			4
#define CMD_MOTOR_SET_PID_SP			5
#define CMD_MOTOR_PID_LOG				6
#define CMD_MOTOR_SET_PARAMS			7
#define CMD_MOTOR_MOVE_V2				8

// CONTROL
#define CMD_CTRL_VISION_POS				0
#define CMD_CTRL_SPLINE_2D				1
#define CMD_CTRL_SPLINE_1D				2
#define CMD_CTRL_SET_FILTER_PARAMS		3
#define CMD_CTRL_SET_PID_PARAMS			4
#define CMD_CTRL_SET_CONTROLLER_TYPE	5
#define CMD_CTRL_VISION_TRACKED_OBJ		6
#define CMD_CTRL_RESET					7

// BASE_STATION
#define CMD_BASE_STATION_ACOMMAND		0
#define CMD_BASE_STATION_PING			1
#define CMD_BASE_STATION_AUTH			2
#define CMD_BASE_STATION_STATS			3
#define CMD_BASE_STATION_CONFIG			4
#define CMD_BASE_STATION_VISION_CONFIG	5
#define CMD_BASE_STATION_WIFI_STATS		6
#define CMD_BASE_STATION_ETH_STATS		7
#define CMD_BASE_STATION_CONFIG_V2		8

// BOOTLOADER
#define CMD_BOOTLOADER_CHECK			0
#define CMD_BOOTLOADER_REQUEST_SIZE		1
#define CMD_BOOTLOADER_REQUEST_CRC32	2
#define CMD_BOOTLOADER_REQUEST_DATA		3
#define CMD_BOOTLOADER_SIZE				4
#define CMD_BOOTLOADER_CRC32			5
#define CMD_BOOTLOADER_DATA				6

// CONFIG
#define CMD_CONFIG_QUERY_FILE_LIST		0
#define CMD_CONFIG_FILE_STRUCTURE		1
#define CMD_CONFIG_ITEM_DESC			2
#define CMD_CONFIG_READ					3
#define CMD_CONFIG_WRITE				4

// STATUS
#define CMD_STATUS_FEEDBACK_PID			0

typedef struct PACKED _ConfigFileStructure
{
	uint16_t id;
	uint16_t version;
	// + data (uint8_t list of items)
} ConfigFileStructure;

typedef struct PACKED _ConfigItemDesc
{
	uint16_t id;
	uint8_t element;	// 0xFF for filename, item description otherwise
	// + zero terminated string
} ConfigItemDesc;

typedef struct PACKED _ConfigReadWrite
{
	uint16_t id;
	// + optional write data
} ConfigReadWrite;

typedef struct PACKED _BootloaderRequestSize
{
	uint8_t procId;
} BootloaderRequestSize;

typedef struct PACKED _BootloaderRequestCRC32
{
	uint8_t procId;
	uint32_t startAddr;
	uint32_t endAddr;
} BootloaderRequestCRC32;

typedef struct PACKED _BootloaderRequestData
{
	uint8_t procId;
	uint32_t offset;
	uint32_t size;
} BootloaderRequestData;

typedef struct PACKED _BootloaderSize
{
	uint8_t procId;
	uint32_t size;
} BootloaderSize;

typedef struct PACKED _BootloaderCRC32
{
	uint8_t procId;
	uint32_t startAddr;
	uint32_t endAddr;
	uint32_t crc;
} BootloaderCRC32;

typedef struct PACKED _BootloaderData
{
	uint8_t procId;
	uint32_t offset;
} BootloaderData;

typedef struct PACKED _KickerChargeAuto	// 2
{
	uint16_t maxCapLevel;
} KickerChargeAuto;

typedef struct PACKED _KickerConfig
{
	uint16_t maxCapLevel;	// [V]
	uint16_t cooldown;		// [ms]
	uint16_t irThreshold;	// [mV]
	uint16_t irFilterLimit;	// number of times the irThreshold is undershot before a interrupted barrier is reported
} KickerConfig;

typedef struct PACKED _SystemAnnouncement	// 12
{
	uint32_t	cpuId[3];
} SystemAnnouncement;

typedef struct PACKED _SystemPing	// 4
{
	uint32_t id;
} SystemPing;

typedef struct PACKED _SystemPong	// 4
{
	uint32_t id;
} SystemPong;

typedef struct PACKED _SystemConsolePrint	// 1 + n
{
	uint8_t source;		// 1
} SystemConsolePrint;

typedef struct PACKED _SystemConsoleCommand	// 1 + n
{
	uint8_t target;		// 1
} SystemConsoleCommand;

#define SYSTEM_MATCH_CTRL_KICK_FLAGS_DEVICE_MASK	0x01
#define SYSTEM_MATCH_CTRL_KICK_FLAGS_MODE_MASK		0x06
#define SYSTEM_MATCH_CTRL_KICK_FLAGS_AUTOCHG_MASK	0x08
#define SYSTEM_MATCH_CTRL_KICK_FLAGS_SEQ_MASK		0xF0

#define SYSTEM_MATCH_CTRL_FLAGS_LIMITED_VEL			0x01
#define SYSTEM_MATCH_CTRL_FLAGS_CHEER				0x02

#define SYSTEM_MATCH_CTRL_UNUSED_FIELD				0x7FFF

#define SYSTEM_MATCH_CTRL_USER_DATA_SIZE			12

typedef struct PACKED _SystemMatchCtrl	// 27
{
	int16_t curPosition[3];	// [mm]				6
	uint8_t posDelay;		// [Q6.2 ms]		1	// maxVal 63.75ms
	uint16_t kickDuration;	// [us]				2
	uint8_t kickFlags;		// mode, dev		1
	int16_t dribblerSpeed;	// [rpm]			2
	uint8_t	skillId;		//					1
	uint8_t flags;			//					1 // cheer, limitedVel
	uint8_t feedbackFreq;	//					1 = 27 B + 4B Header/COBS = 31B
	uint8_t skillData[SYSTEM_MATCH_CTRL_USER_DATA_SIZE];	// skill dependent	12
} SystemMatchCtrl;

#define SYSTEM_MATCH_FEEDBACK_UNUSED_FIELD			0x7FFF
#define SYSTEM_MATCH_FEEDBACK_BKC_BARRIER_MASK		0x80
#define SYSTEM_MATCH_FEEDBACK_BKC_KICK_MASK			0x7F

#define SYSTEM_MATCH_FEEDBACK_FEATURE_MOVE			0x0001
#define SYSTEM_MATCH_FEEDBACK_FEATURE_DRIBBLE		0x0002
#define SYSTEM_MATCH_FEEDBACK_FEATURE_STRAIGHT		0x0004
#define SYSTEM_MATCH_FEEDBACK_FEATURE_CHIP			0x0008
#define SYSTEM_MATCH_FEEDBACK_FEATURE_BARRIER		0x0010
#define SYSTEM_MATCH_FEEDBACK_NUM_FEATURES			5

typedef struct PACKED _SystemMatchFeedback	// 22
{
	int16_t curPosition[3];		// [mm]			6
	int16_t curVelocity[3];		// [mm/s]		6
	uint8_t kickerLevel;		// [V]			1
	int16_t dribblerSpeed;		// [rpm]		2
	uint16_t batteryLevel;		// [mV]			2
	uint8_t barrierKickCounter;	//				1
	uint16_t features;			//				2	movement, dribbler, barrier, straight, chip
	uint8_t hardwareId;			//				1
	uint8_t dribblerTemp;		//				1
} SystemMatchFeedback;

#define SYSTEM_QUERY_PID_PARAMS			0
#define SYSTEM_QUERY_STRUCTURE			1
#define SYSTEM_QUERY_EZ_SENSOR			2
#define SYSTEM_QUERY_EX_STATE			3
#define SYSTEM_QUERY_CONTROLLER_TYPE	4
#define SYSTEM_QUERY_KICKER_CONFIG		5

typedef struct PACKED _SystemQuery
{
	uint16_t parameterId;
} SystemQuery;

typedef struct PACKED _SystemPerformance	// 12
{
	uint16_t accMax;	// [mm/s^2]
	uint16_t accMaxW;	// [crad/s^2]
	uint16_t brkMax;	// [mm/s^2]
	uint16_t brkMaxW;	// [crad/s^2]
	uint16_t velMax;	// [mm/s]
	uint16_t velMaxW;	// [crad/s]
} SystemPerformance;

typedef struct PACKED _StatusFeedbackPid // 20
{
	int16_t t;
	int16_t in[3];
	int16_t out[3];
	int16_t set[3];
} StatusFeedbackPid;


#define CTRL_FILTER_PARAM_EX_POS		0x01
#define CTRL_FILTER_PARAM_EX_VEL		0x02
#define CTRL_FILTER_PARAM_EX_ACC		0x03
#define CTRL_FILTER_PARAM_EZ_VISION		0x04
#define CTRL_FILTER_PARAM_EZ_ENCODER	0x05
#define CTRL_FILTER_PARAM_EZ_ACC_GYRO	0x06
#define CTRL_FILTER_PARAM_EZ_MOTOR		0x07

typedef struct PACKED _CtrlSetFilterParams // 13
{
	float params[3];	// X, Y, W
	uint8_t type;
} CtrlSetFilterParams;

#define CTRL_PID_PARAM_POS_X 0x01
#define CTRL_PID_PARAM_POS_Y 0x02
#define CTRL_PID_PARAM_POS_W 0x03
#define CTRL_PID_PARAM_VEL_X 0x04
#define CTRL_PID_PARAM_VEL_Y 0x05
#define CTRL_PID_PARAM_VEL_W 0x06
#define CTRL_PID_PARAM_MOTOR 0x07

typedef struct PACKED _CtrlSetPIDParams // 13
{
	float Kp;
	float Ki;
	float Kd;
	uint8_t type;
} CtrlSetPIDParams;

//#define CTRL_TYPE_FUSION_VEL	1 // obsolete
#define CTRL_TYPE_FUSION		2
#define CTRL_TYPE_MOTOR			3
#define CTRL_TYPE_CALIBRATE		4
#define CTRL_TYPE_TIGGA			5

typedef struct PACKED _CtrlSetControllerType // 1
{
	uint8_t controllerId;
} CtrlSetControllerType;

typedef struct PACKED _BaseStationACommand
{
	uint8_t id;
	uint8_t* pCommand;
} BaseStationACommand;

typedef struct PACKED _BaseStationPing
{
	uint32_t id;
	uint8_t* pData;
} BaseStationPing;

typedef struct PACKED _BaseStationAuth	// 4
{
	uint32_t key;	// 0x42424242
} BaseStationAuth;

typedef struct PACKED _BaseStationConfigV2 // 16
{
	uint8_t visionIp[4];
	uint16_t visionPort;

	struct _module
	{
		uint8_t channel;		// = frequency
		uint8_t speed;			// 0 = 250k, 1 = 1M, 2 = 2M
		uint8_t maxBots;		// 1 - 24
		uint8_t fixedRuntime;
		uint32_t timeout;		// in [ms]
	} module[2];
} BaseStationConfigV2;

typedef struct PACKED _BaseStationWifiStats	// 624
{
	struct _bot
	{
		uint8_t botId;

		uint8_t queueStats[26];
	} bots[24];

	uint16_t updateRate;	// [Hz]
} BaseStationWifiStats;

typedef struct PACKED _BaseStationEthStats // 18
{
	uint32_t txFrames;
	uint32_t txBytes;

	uint32_t rxFrames;
	uint32_t rxBytes;

	uint16_t rxFramesDmaOverrun;
} BaseStationEthStats;

#endif /* COMMANDS_H_ */
