/*
 * crc.c
 *
 *  Created on: 06.06.2013
 *      Author: AndreR
 */

#include "util/crc.h"

#include "stm32f407xx.h"
#include "ch.h"

static Mutex crcMtx;

void CRCInit()
{
	chMtxInit(&crcMtx);

	RCC->AHB1ENR |= RCC_AHB1ENR_CRCEN;
}

uint32_t CRC32Checksum(uint32_t* pData, uint32_t dataLength)
{
	chMtxLock(&crcMtx);

	CRC->CR |= CRC_CR_RESET;

	uint32_t index = 0;

	for(index = 0; index < dataLength; index++)
	{
		CRC->DR = __RBIT(pData[index]);
	}

	uint32_t result = (__RBIT(CRC->DR) ^ 0xFFFFFFFF);

	chMtxUnlock();

	return result;
}

void CRC32FeedChecksum(uint8_t* pData, uint32_t dataLength)
{
	uint32_t index;

	for(index = 0; index < dataLength-3; index += 4)
	{
		CRC->DR = __RBIT(*((uint32_t*)&pData[index]));
	}

	uint32_t lastWord = 0;
	uint32_t bytesLeft = dataLength-index;

	if(bytesLeft > 2)
		lastWord |= pData[index+2] << 16;
	if(bytesLeft > 1)
		lastWord |= pData[index+1] << 8;
	if(bytesLeft > 0)
	{
		lastWord |= pData[index];
		CRC->DR = __RBIT(lastWord);
	}
}

void CRC32StartChecksum()
{
	chMtxLock(&crcMtx);

	CRC->CR |= CRC_CR_RESET;
}

uint32_t CRC32StopChecksum()
{
	uint32_t result = (__RBIT(CRC->DR) ^ 0xFFFFFFFF);

	chMtxUnlock();

	return result;
}

uint32_t CRC32CalcChecksum(uint8_t* pData, uint32_t dataLength)
{
	CRC32StartChecksum();

	CRC32FeedChecksum(pData, dataLength);

	return CRC32StopChecksum();
}

uint32_t CRC32VerifyChecksum(uint8_t* pData, uint32_t dataLength, uint32_t crc)
{
	CRC32StartChecksum();

	CRC32FeedChecksum(pData, dataLength);

	CRC->DR = __RBIT(crc);

	return CRC32StopChecksum();
}
