/*
 * spi_sync.h
 *
 *  Created on: 28.09.2013
 *      Author: AndreR
 */

#ifndef __SPI_SYNC_H__
#define __SPI_SYNC_H__

#include "stm32f407xx.h"

#include "ch.h"

typedef struct _SPISyncData
{
	SPI_TypeDef* pRegister;

	DMA_TypeDef* pDma;
	uint16_t dmaChTx;
	uint16_t dmaChRx;
	uint32_t dmaPrio;

	GPIO_TypeDef* pCSPort;
	uint16_t csPin;

	uint8_t prescaler;
	uint8_t cpol;
	uint8_t cpha;

	uint32_t timeoutTicks;
} SPISyncData;

typedef struct _SPISync
{
	SPISyncData data;

	uint8_t dummy;

	DMA_Stream_TypeDef* pDmaTx;
	DMA_Stream_TypeDef* pDmaRx;

	__IO uint32_t* pDMARxIFCR;
	__IO uint32_t* pDMATxIFCR;
	uint32_t IFCRRxMask;
	uint32_t IFCRTxMask;

	BinarySemaphore transferSem;
	Mutex taskMutex;
} SPISync;

void SPISyncDmaRxInterrupt(SPISync* pSPI);
void SPISyncInit(SPISync* pSPI, SPISyncData* pInit);
void SPISyncStartTransfer(SPISync* pSPI, uint8_t* pSrc, uint8_t* pDst, uint32_t size);
int16_t SPISyncWait(SPISync* pSPI);
int16_t SPISyncTransfer(SPISync* pSPI, uint8_t* pSrc, uint8_t* pDst, uint32_t size);

#endif
