/*
 * usart.h
 *
 *  Created on: 04.11.2012
 *      Author: AndreR
 */

#ifndef USART_H_
#define USART_H_

#include "stm32f407xx.h"

#include "util/fifo_lin.h"

#include "ch.h"

typedef void(*StartTimerFunc)(uint32_t usec);

typedef struct _USARTData
{
	uint8_t* pDataTx;
	uint16_t txSize;
	uint8_t* pDataRx;
	uint16_t rxSize;

	uint32_t maxPacketSize;

	uint32_t dmaChannel;
	uint32_t rxStream;
	uint32_t txStream;

	DMA_TypeDef* pDMA;

	uint8_t rxIRQn;
	uint8_t txIRQn;
	uint8_t usartIRQn;
	uint8_t IRQL;

	uint32_t baudrate;

	StartTimerFunc pStartTimer;
} USARTData;

typedef void(*USARTCallback)(void*);

typedef struct _USART
{
	USART_TypeDef* pRegister;

	USARTData data;

	FifoLin txFifo;
	FifoLin rxFifo;

	DMA_Stream_TypeDef* pRxStream;
	DMA_Stream_TypeDef* pTxStream;

	volatile uint32_t* pDMARxIFCR;
	volatile uint32_t* pDMATxIFCR;
	volatile uint32_t* pDMARxISR;
	volatile uint32_t* pDMATxISR;
	uint32_t IFCRRxMask;
	uint32_t IFCRTxMask;

	USARTCallback callback;
	void* pUserData;

	volatile uint8_t txState;
	volatile uint8_t rxState;

	systime_t txTime;
	systime_t rxWaitStartTime;

	Mailbox eventQueue;
	msg_t eventQueueData[4];

	struct
	{
		uint32_t rxInvalid;
	} stats;
} USART;

void	USARTInit(USART* pUSART, USART_TypeDef* pRegister, USARTData data);
void	USARTProcess(USART* pUSART);
msg_t	USARTTask(void* params);
int16_t USARTSend(USART* pUSART, uint16_t length, uint8_t* pData);
int16_t USARTSendConcat(USART* pUSART, uint16_t length, uint8_t* pData, uint16_t length2, uint8_t* pData2);
int16_t USARTStartTransfer(USART* pUSART);

// IRQ functions
void 	USARTIRQ(USART* pUSART);
void	USARTTimerIRQ(USART* pUSART);

#endif
