/*
 * intercom_sections.h
 *
 *  Created on: 11.11.2012
 *      Author: AndreR
 */

#ifndef INTERCOM_SECTIONS_H_
#define INTERCOM_SECTIONS_H_

#include "commands.h"

// Sections
//#define INTERCOM_SECTION_CONTROL		0x01
#define INTERCOM_SECTION_CONSOLE		0x02
#define INTERCOM_SECTION_BOOTLOADER		0x04
#define INTERCOM_SECTION_PRESENTER		0x05
#define INTERCOM_SECTION_SUPPLY			0x06
#define INTERCOM_SECTION_KD				0x07
#define INTERCOM_SECTION_COMMAND		0x08
#define INTERCOM_SECTION_NETWORK		0x09
#define INTERCOM_SECTION_MOTORS			0x0A
#define INTERCOM_SECTION_UI				0x0B
#define INTERCOM_SECTION_LOGFILE		0x0C
#define INTERCOM_SECTION_CONFIG			0x0D
#define INTERCOM_SECTION_RELIABLE		0x80

// Addresses
#define INTERCOM_ADDRESS_MAIN	0x01
#define INTERCOM_ADDRESS_MEDIA	0x02
#define INTERCOM_ADDRESS_LEFT	0x04
#define INTERCOM_ADDRESS_RIGHT	0x08
#define INTERCOM_ADDRESS_KD		0x10

#define INTERCOM_ID_MAIN		0
#define INTERCOM_ID_MEDIA		1
#define INTERCOM_ID_LEFT		2
#define INTERCOM_ID_RIGHT		3
#define INTERCOM_ID_KD			4

#define INTERCOM_MAX_ADDRESSES 8

// RELIABLE
#define IC_RELIABLE_DATA		0
#define IC_RELIABLE_ACK			1

// BOOTLOADER
#define BOOTLOADER_SELECT	0
#define BOOTLOADER_ERASE	1
#define BOOTLOADER_DATA		2
#define BOOTLOADER_RESPONSE	3
#define BOOTLOADER_CHECKSUM 5
#define BOOTLOADER_ACK		6
#define BOOTLOADER_NACK		7

// CONFIG PRESENTER
#define IC_PRESENTER_CONFIG			0
#define IC_PRESENTER_KICKER_STATUS	1
#define IC_PRESENTER_DISCHARGE		2
#define IC_PRESENTER_POWER			3
#define IC_PRESENTER_CFG_MISC		4
#define IC_PRESENTER_WIFI_STAT		5

// NETWORK
#define IC_NETWORK_NORMAL	0
#define IC_NETWORK_RELIABLE	1

// UI
#define IC_UI_WIFI_BOTID	0
#define IC_UI_WIFI_SETTINGS	1
#define IC_UI_WIFI_SAVE		2
#define IC_UI_FEATURES		3

// Messages
#define IC_SUPPLY_POWER_LOG	0

#define IC_KD_FEEDBACK		0
#define IC_KD_MATCH_COMMAND	1
#define IC_KD_AUTOCHG		2
#define IC_KD_CONFIG		3

#define IC_MOTORS_MAIN2MOT 	0
#define IC_MOTORS_MOT2MAIN	1
#define IC_MOTORS_TEMP		2
#define IC_MOTORS_CTRL_TEMP_MEAS 3

#define IC_LOGFILE_TIMESYNC	0
#define IC_LOGFILE_MESSAGE	1

typedef struct PACKED _KdFeedback
{
	struct
	{
		float cap;	// [V]
		float chg;	// [A]
		uint16_t state;	// bitfield
		uint8_t kickCounter;
	} kicker;

	struct
	{
		float velocity;	// [rpm]
		float temp;		// [C]
		uint8_t overheated;
	} dribbler;

	struct
	{
		float bat;	// [V]
		float cur;	// [A]
		uint8_t batEmpty;
	} power;

	struct
	{
		uint16_t level[2];	// IR ADC average
		uint8_t interrupted;
	} barrier;

	KickerConfig config;
} KdFeedback;

typedef struct PACKED _KdCommand
{
	uint8_t seq;
	uint8_t autoChg;
	uint8_t mode;
	uint8_t device;
	float duration;	// [s]
	float dribblerSpeed;	// [rpm]
} KdCommand;

typedef struct PACKED _LogFileTimeSync
{
	uint32_t time;
} LogFileTimeSync;

typedef struct PACKED _SupplyPowerLog
{
	float bat;	// [V]
	float cur;	// [A]
} SupplyPowerLog;

typedef struct PACKED _IrLog
{
	uint16_t level[2];	// IR ADC average
	uint8_t interrupted;
} IrLog;

typedef struct PACKED _KickerLog
{
	float vCap;	// [V]
	float iChg;	// [A]
	uint16_t state;	// bitfield
} KickerLog;

typedef struct PACKED _ConfigNetwork
{
	uint8_t botId;
	uint8_t channel;		// frequency
	uint8_t speed;
} ConfigNetwork;

typedef struct _ConfigMisc
{
	uint16_t features;
	uint8_t hardwareId;
} ConfigMisc;

typedef struct PACKED _UIWifiBotID
{
	uint8_t id;
} UIWifiBotID;

typedef struct PACKED _UIWifiSettings
{
	uint8_t speed;
	uint8_t channel;
} UIWifiSettings;

typedef struct PACKED _PresenterWifiStat
{
	uint8_t sumatraOnline;
	uint8_t bsOnline;
	uint16_t updateFreq;	// [Hz]
	uint16_t visionDelay;	// [us]
} PresenterWifiStat;

typedef struct PACKED _ReliableDataAck
{
	uint16_t seq;
	uint8_t srcAddr;
} ReliableDataAck;

typedef struct PACKED _BootloaderChecksum
{
	uint16_t seq;
	uint32_t start;
	uint32_t end;
	uint32_t crc;
} BootloaderChecksum;

typedef struct PACKED _BootloaderSeq
{
	uint16_t seq;
} BootloaderSeq;

typedef struct PACKED _BootloaderProgramData
{
	uint16_t seq;
	uint32_t offset;
} BootloaderProgramData;

//################### MOTORS ####################
#define MOT_FRONT	0
#define MOT_REAR 	1
#define MOT_SOURCE_RIGHT 	0
#define MOT_SOURCE_LEFT		1
#define MOT_CTRL_OFF		0
#define MOT_CTRL_VELOCITY	1
#define MOT_CTRL_VOLTAGE	2
#define MOT_CTRL_TORQUE		3

typedef struct PACKED _MotorSetpoint
{
	float velocity; 	// motor rounds per second
	float voltage;		// motor [V]
	float torque;		// motor [Nm]
} MotorSetpoint;

typedef struct PACKED _MotorsMain2Mot
{
	MotorSetpoint mot[2];

	float bat;	// [V]
	uint8_t ctrlMode;
} MotorsMain2Mot;

typedef struct PACKED _MotorFeedback
{
	float velocity;
	float rmsCurrent;
	float torque;

	uint32_t velocityTimestamp;
	uint32_t rmsCurrentTimestamp;
	uint32_t torqueTimestamp;

	float setVoltage;
	uint32_t setVoltageTimestamp;
} MotorFeedback;

typedef struct PACKED _MotorsMot2Main
{
	uint8_t source;	// MOT_LEFT or MOT_RIGHT

	MotorFeedback mot[2];
} MotorsMot2Main;

typedef struct PACKED _MotorsTemp
{
	uint8_t left;
	uint8_t rear;

	float t[3];
} MotorsTemp;

typedef struct PACKED _MotorsCtrlTempMeas
{
	uint8_t enable;
} MotorsCtrlTempMeas;

#endif /* INTERCOM_SECTIONS_H_ */
