/*
 * console_intercom_adapter.c
 *
 *  Created on: 16.11.2012
 *      Author: AndreR
 */

#include "console_intercom_adapter.h"

#include "intercom_constants.h"
#include "intercom_reliable.h"

#include <string.h>

static void intercomCallback(const IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	(void)pHeader;
	uint8_t* pConData;

	ConsoleIntercomAdapter* pAdapter = (ConsoleIntercomAdapter*)pUserData;

	int16_t result = FifoLinReserve(&console.rxFifo, dataLength+1, &pConData);
	if(result)
	{
		pAdapter->droppedPackets++;
		return;
	}

	memcpy(pConData, pData, dataLength);

	pConData[dataLength] = '\0';

	FifoLinCommit(&console.rxFifo, dataLength+1);
}

static void consoleTxCallback(void* pUserData)
{
	uint32_t size;
	uint8_t* pData;
	int16_t result;

	ConsoleIntercomAdapter* pAdapter = (ConsoleIntercomAdapter*)pUserData;

	IntercomHeader header;
	header.address = pAdapter->intercomDestination;
	header.command = pAdapter->intercomSource;
	header.section = INTERCOM_SECTION_CONSOLE;

	chMtxLock(&pAdapter->conTxMutex);

	while(FifoLinGet(&console.txFifo, &pData, &size) == 0)
	{
		result = IntercomReliableSend(&header, size, pData);
		if(result)
			pAdapter->droppedPackets++;

		FifoLinDelete(&console.txFifo);
	}

	chMtxUnlock();
}

void ConsoleIntercomAdapterInit(ConsoleIntercomAdapter* pAdapter, uint8_t source, uint8_t dest)
{
	pAdapter->droppedPackets = 0;
	pAdapter->intercomSource = source;
	pAdapter->intercomDestination = dest;

	chMtxInit(&pAdapter->conTxMutex);

	IntercomClient client;
	client.pUserData = pAdapter;
	client.rxCallback = &intercomCallback;
	client.section = INTERCOM_SECTION_CONSOLE;

	IntercomAddClient(client);

	console.data.txCallback = &consoleTxCallback;
	console.data.pUserData = pAdapter;
}
