/*
 * flash_fs.h
 *
 *  Created on: 25.06.2014
 *      Author: AndreR
 */

#ifndef FLASH_FS_H_
#define FLASH_FS_H_

#include <stdint.h>
#include "ch.h"

#define FLASH_TEMP_SIZE 4096

#define FLASH_FS_OPEN 0
#define FLASH_FS_CREATED 1
#define FLASH_FS_NOMEM 2
#define FLASH_FS_SIZE_MISMATCH 3

typedef struct _FlashFile
{
	uint16_t blockSize;
	uint16_t headerSize;	// sizeof(FlashFile) + strlen(filename) + (padding to 4 byte)
	uint16_t dataSize;
	uint16_t version;
	uint32_t del;
} __attribute__((aligned(4),packed)) FlashFile;

typedef struct _FlashFS
{
	uint32_t flashAddr;
	uint32_t flashSize;
	uint32_t bytesDel;
	uint32_t bytesUsed;

	uint8_t* pWrite;
	uint8_t* pEnd;

	uint8_t temp[FLASH_TEMP_SIZE];

	Mutex writeMutex;

	uint32_t version;
} FlashFS;

extern FlashFS flashFS;

void	FlashFSInit(uint32_t flashAddr, uint32_t flashSize);
FlashFile* FlashFSOpen(const char* pFilename, uint16_t version);
FlashFile* FlashFSCreate(const char* pFilename, uint16_t version, uint16_t size);

/**
 * Open an existing file on the FlashFS or create a new one.
 *
 * If the file exists, its content is copied to pData.
 * If the file does not exist, it is filled with pData.
 */
int16_t	FlashFSOpenOrCreate(const char* pFilename, uint16_t version, void* pData, uint16_t size, FlashFile** ppFH);
void	FlashFSRead(FlashFile* pFH, void* pDst);

/**
 * Write pData to the FlashFS.
 *
 * @note No data is written if the file content equals pData.
 */
int16_t	FlashFSWrite(FlashFile** ppFH, void* pData, uint16_t size);
void	FlashFSDelete(const char* pFilename);
int16_t	FlashFSCompact();
void 	FlashFSListFiles();

#endif /* FLASH_FS_H_ */
