/*
 * intercom.h
 *
 *  Created on: 10.11.2012
 *      Author: AndreR
 */

#ifndef INTERCOM_H_
#define INTERCOM_H_

#include <stdint.h>

#include "util/fifo_lin.h"

#define INTERCOM_MAX_CLIENTS	16
#define INTERCOM_MAX_MEDIA		4

#define INTERCOM_HEADER_SIZE 3
#define INTERCOM_TRAILER_SIZE 4	// CRC32

typedef union __attribute__ ((packed)) _IntercomHeader
{
	struct
	{
		uint8_t address;
		uint8_t section;
		uint8_t command;
	};

	uint8_t _data[INTERCOM_HEADER_SIZE];
} IntercomHeader;

typedef void(*IntercomCallback)(const IntercomHeader*, uint16_t, uint8_t*, void*);	//header, dataLength, data, userData

typedef struct _IntercomClient
{
	uint8_t section;
	IntercomCallback rxCallback;
	void* pUserData;
} IntercomClient;

typedef void(*IntercomTxReady)(void*);

typedef struct _IntercomMedium
{
	FifoLin* pTxFifo;
	FifoLin* pRxFifo;

	IntercomTxReady txReady;
	void* pTxReadyData;

	uint8_t reachableAddresses;
} IntercomMedium;

typedef struct _Intercom
{
	IntercomMedium media[INTERCOM_MAX_MEDIA];
	uint8_t numMediaUsed;

	IntercomClient clients[INTERCOM_MAX_CLIENTS];
	uint8_t numClientsUsed;

	uint8_t ownAddress;
	uint8_t ownId;
	uint8_t hub;

	BinarySemaphore rxReady;
} Intercom;

extern Intercom intercom;

void	IntercomInit(uint8_t ownAddr, uint8_t hub);
int16_t IntercomAddMedium(IntercomMedium medium);
void 	IntercomRxReady(void* pUserData);
int16_t IntercomSend(const IntercomHeader* pHeader, uint16_t dataLength, void* _pData);
int16_t IntercomAddClient(IntercomClient client);
msg_t	IntercomTask(void* params);
void	IntercomExecRxCallback(const IntercomHeader* pHeader, uint8_t* pData, uint32_t dataLength);

#endif /* INTERCOM_H_ */
