/*
 * prog_client.c
 *
 *  Created on: 03.04.2015
 *      Author: AndreR
 */

#include "prog_client.h"

#include "intercom_constants.h"
#include "util/intercom.h"
#include "util/boot.h"
#include "util/crc.h"

ProgClient progClient;

static void sendNACK(uint16_t seq)
{
	BootloaderSeq bSeq = {seq};

	IntercomHeader header;
	header.address = INTERCOM_ADDRESS_MEDIA;
	header.section = INTERCOM_SECTION_BOOTLOADER;
	header.command = BOOTLOADER_NACK;

	IntercomSend(&header, sizeof(BootloaderSeq), &bSeq);
}

static void sendACK(uint16_t seq)
{
	BootloaderSeq bSeq = {seq};

	IntercomHeader header;
	header.address = INTERCOM_ADDRESS_MEDIA;
	header.section = INTERCOM_SECTION_BOOTLOADER;
	header.command = BOOTLOADER_ACK;

	IntercomSend(&header, sizeof(BootloaderSeq), &bSeq);
}

static void progClientCb(const IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	(void)dataLength;
	(void)pUserData;

	switch(pHeader->command)
	{
		case BOOTLOADER_ERASE:
		{
			BootloaderSeq* pSeq = (BootloaderSeq*)pData;

			if(progClient.programCodeErased)
				sendACK(pSeq->seq);

			if(BootEraseProgramFlash())
			{
				sendNACK(pSeq->seq);
			}
			else
			{
				sendACK(pSeq->seq);
				progClient.programCodeErased = 1;
			}
		}
		break;
		case BOOTLOADER_DATA:
		{
			BootloaderProgramData* pProgData = (BootloaderProgramData*)pData;
			pData += sizeof(BootloaderProgramData);
			dataLength -= sizeof(BootloaderProgramData);

			if((dataLength % sizeof(uint32_t)) != 0)
				sendNACK(pProgData->seq);

			if(BootProgramFlash((uint32_t*)pData, dataLength/4, pProgData->offset))
				sendNACK(pProgData->seq);
			else
				sendACK(pProgData->seq);
		}
		break;
		case BOOTLOADER_CHECKSUM:
		{
			BootloaderChecksum* pReq = (BootloaderChecksum*)pData;

			if(CRC32VerifyChecksumAlt((uint8_t*)(BOOT_APP_ADDR+pReq->start), pReq->end-pReq->start, pReq->crc) == CRC32_MAGIC_NUMBER)
				sendACK(pReq->seq);
			else
				sendNACK(pReq->seq);
		}
		break;
		case BOOTLOADER_SELECT:
		{
			BootSetBootloaderSelected(1);
		}
		break;
	}
}

void ProgClientInit()
{
	IntercomClient client;
	client.pUserData = 0;
	client.section = INTERCOM_SECTION_BOOTLOADER;
	client.rxCallback = &progClientCb;

	IntercomAddClient(client);
}
