/*
 * sys_time.c
 *
 *  Created on: 25.10.2014
 *      Author: AndreR
 */

#include "sys_time.h"
#include "intercom_constants.h"

#define IN_CCM_VAR __attribute__((section(".ccmvar")))

#define NVIC_SYSTICK_CNT ( ( volatile unsigned long * ) 0xE000E018 )

#ifdef STM32F4XX
#define CPU_CLOCK_HZ 168000000UL
#define IN_CCM
#else
#define CPU_CLOCK_HZ 72000000UL
#define IN_CCM __attribute__((section(".ccmtext")))
#endif
#define F_CPU_CLOCK_HZ ((float)CPU_CLOCK_HZ)
#define F_TICK_RATE_HZ ((float)CH_FREQUENCY)
#define SYSTICK_CNT_MAX ((CPU_CLOCK_HZ / CH_FREQUENCY) - 1UL)
#define CNT_CLK_1MHZ (CPU_CLOCK_HZ/1000000UL)
#define USEC_PER_TICK (1000000UL/CH_FREQUENCY)

IN_CCM_VAR SysTimeGlobal sysTime;

static void logCallback(const IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	(void)dataLength;
	(void)pUserData;

	const float alpha = 0.975f;

	if(pHeader->section != INTERCOM_SECTION_LOGFILE || pHeader->command != IC_LOGFILE_TIMESYNC)
		return;

	uint32_t myTime = SysTimeUSecRaw();
	LogFileTimeSync* pSync = (LogFileTimeSync*)pData;
	int32_t corr = pSync->time-myTime+sysTime.comOffset;
	if(sysTime.correction == 0)
		sysTime.correction = corr;
	else
		sysTime.correction = (alpha*sysTime.correction + (1.0f-alpha)*corr);

	sysTime.lastCorrTime = chTimeNow();
}

void SysTimeInitCorrection(int32_t transmissionTime)
{
	sysTime.comOffset = transmissionTime;

	IntercomClient iClient;
	iClient.rxCallback = &logCallback;
	iClient.pUserData = 0;
	iClient.section = INTERCOM_SECTION_LOGFILE;

	IntercomAddClient(iClient);
}

IN_CCM unsigned long SysTimeUSecRaw()
{
	return chTimeNow()*USEC_PER_TICK + (SYSTICK_CNT_MAX-*NVIC_SYSTICK_CNT)/CNT_CLK_1MHZ; // [us]
}

// microsecond precision, integer arithmetic => fast + recommended
IN_CCM unsigned long SysTimeUSec()
{
	return SysTimeUSecRaw()+sysTime.correction; // [us]
}

// converts SysTimeUSec to seconds (float)
float SysTime()
{
	return SysTimeUSec()*1e-6f;
}

// maximum precision, uses floating point arithmetic => slow
float SysTimePrec()
{
	return chTimeNow()/F_TICK_RATE_HZ + (SYSTICK_CNT_MAX-*NVIC_SYSTICK_CNT)/F_CPU_CLOCK_HZ + sysTime.correction;
}
