clc

% ON pulse data taken from IRS2336 datasheet
% PWMin, diff, PWMout
on = [350, 107, 457
400, 88, 488
450, 72, 522
500, 60, 560
550, 47, 597
600, 40, 640
650, 30, 680
700, 26, 726
750, 19, 769
800, 18, 818
850, 11, 861
900, 11, 911
950, 8, 958
1000, 9, 1009];

y = on(:,2);
x = on(:,3);

[fitPow, fitPowGOF] = fit(x, y, 'power2');

nSamples = 3600;

Tpwm = 1/20e3;
Tres = Tpwm/nSamples;

Tres = Tres*1e9;
Tend = round(1000/Tres);

samples = zeros(nSamples, 1);
time = zeros(nSamples, 1);
corr = zeros(nSamples, 1);
lowerBound = ones(nSamples, 1)*350;

for i = 1:nSamples
    time(i) = i*Tres;
    if time(i) < 460
        samples(i) = 0;
    elseif time(i) > 1000
        samples(i) = time(i);
    else
        samples(i) = time(i)-feval(fitPow, time(i));
    end
    
    corr(i) = round(samples(i)/Tres);
end

plot(time, samples, time, time, time, lowerBound);
plot(1:Tend, corr(1:Tend), 1:Tend, 1:Tend);
fprintf('#define DRIVE_NUM_CORR = %d\n', Tend);
fprintf('const uint16_t ccrCorr[DRIVE_NUM_CORR] = {\n', Tend);
fprintf('%d, ', corr(1:Tend));
fprintf('};');
fprintf('\n');