function [ logData, in, out, sampleTimes, pid] = LogFileViewer()
%LOGFILEVIEWER Graphical viewer for log file data
%   Awesome tool for data analysis

clc;
clear;

global state;
global filename;
global pathname;
% [filename, pathname] = uigetfile('*.dat');
% logData = loadLogFile(fullfile(pathname, filename));

%filename = '../move_rotate.dat';
%filename = '../move_straight.dat';
%filename = '../move_side.dat';
%logData = loadLogFile(filename);

% [in, state, out, sampleTimes] = processCtrlData(logData);
% [ pid ] = processPidData( logData );

figure('Name', strcat('Logfile Viewer: ', filename), 'NumberTitle', 'Off');
m = uimenu('Label', 'Dataset');
uimenu(m, 'Label', 'Load File', 'Callback', @loadFile);
uimenu(m, 'Label', 'Global Position', 'Callback', @showGlobalPosition);
uimenu(m, 'Label', 'Local Position (Y of X)', 'Callback', @showLocalPositionXY);
uimenu(m, 'Label', 'Local Position (XYW of t)', 'Callback', @showLocalPositionXYW);
uimenu(m, 'Label', 'Local Velocity', 'Callback', @showLocalVelocity);
uimenu(m, 'Label', 'Local Velocity 2', 'Callback', @showLocalVelocity2);
uimenu(m, 'Label', 'Motor Velocity', 'Callback', @showMotorVelocities);
uimenu(m, 'Label', 'Motor Voltages', 'Callback', @showMotorVoltages);
uimenu(m, 'Label', 'Acceleration', 'Callback', @showAcceleration);
uimenu(m, 'Label', 'Accel. Velocity', 'Callback', @showAccelerometerVelocity);
uimenu(m, 'Label', 'Gyro FFT', 'Callback', @showGyrFFT);
uimenu(m, 'Label', 'Acc. FFT', 'Callback', @showAccFFT);
uimenu(m, 'Label', 'PIDs', 'Callback', @showPID);
uimenu(m, 'Label', 'trajectory', 'Callback', @showTrajectory);

loadFile(0,0);

function loadFile(~, ~)
    [filename, pathname] = uigetfile('*.dat');
    logData = loadLogFile(fullfile(pathname, filename));
    [in, state, out, sampleTimes] = processCtrlData(logData);
    [ pid ] = processPidData( logData );
    showLocalPositionXY(0, 0);
end

function showLocalPositionXY(~, ~)
    subplot(1, 1, 1);
    plot(in.pos.enc.local(:,1), in.pos.enc.local(:,2), ...
        in.pos.vis.local(:,1), in.pos.vis.local(:,2));
    axis equal
    legend('Encoder', 'Vision');
    title('Local Position (Vision+Encoder)');
    xlabel('X [m]');
    ylabel('Y [m]');
end

function showLocalPositionXYW(~, ~)
	subplot(2, 1, 1);
	plot(sampleTimes, in.pos.enc.local(:,1), 'b-', sampleTimes, in.pos.vis.local(:,1), 'c:', ...
		sampleTimes, in.pos.enc.local(:,2), 'r-', sampleTimes, in.pos.vis.local(:,2), 'm:');
	legend('X Enc', 'X Vision', 'Y Enc', 'Y Vision');
	title('XY Position (Vision+Encoders)');
	xlabel('t [s]');
	ylabel('s [m]');
	axis tight

    subplot(2, 1, 2);
    plot(sampleTimes, in.pos.enc.local(:,3), sampleTimes, in.pos.vis.local(:,3), ...
		sampleTimes, in.pos.gyr.local(:,3))
    axis tight
    legend('Encoder', 'Vision', 'Gyro');
    title('Local Orientation');
    xlabel('t [s]');
    ylabel('w [rad]');
end

function showMotorVelocities(~, ~)
	vmin = min(min(in.mot.vel));
	vmax = max(max(in.mot.vel));
	vlim = [vmin vmax];
	
    subplot(2, 2, 2);
    plot(sampleTimes, in.mot.vel(:,1), sampleTimes, out.mot.vel(:,1));
    legend('Real', 'Setpoint');
    title('M1 (Front Right)');
    ylabel('v [Motor RPS]');
    xlabel('t [s]');
	axis tight
	ylim(vlim);    

    subplot(2, 2, 1);
    plot(sampleTimes, in.mot.vel(:,2), sampleTimes, out.mot.vel(:,2));
    title('M2 (Front Left)');
    ylabel('v [Motor RPS]');
    xlabel('t [s]');
	axis tight
	ylim(vlim);

    subplot(2, 2, 3);
    plot(sampleTimes, in.mot.vel(:,3), sampleTimes, out.mot.vel(:,3));
    title('M3 (Rear Left)');
    ylabel('v [Motor RPS]');
    xlabel('t [s]');
	axis tight
	ylim(vlim);

    subplot(2, 2, 4);
    plot(sampleTimes, in.mot.vel(:,4), sampleTimes, out.mot.vel(:,4));
    title('M4 (Rear Right)');
    ylabel('v [Motor RPS]');
    xlabel('t [s]');
	axis tight
	ylim(vlim);
end

function showMotorVoltages(~, ~)
	vmin = min(min(in.mot.vel));
	vmax = max(max(in.mot.vel));
	vlim = [vmin-10 vmax+10];
    umin = min(min(out.mot.vol));
    umax = max(max(out.mot.vol));
    
    uF = (vmax-vmin)/(umax-umin);
    
    PT1 = zeros(4,2);
    PT1fit = zeros(1001, 8);
    for i = 1:4
        iStart = find(in.mot.vel(:,i)>0, 1);
        slice = [sampleTimes(iStart:iStart+1000), in.mot.vel(iStart:iStart+1000,i)];
        offsetT = slice(1,1);
        slice(:,1) = slice(:,1)-offsetT;

        K = mean(slice(800:end,2));
        tT = find(slice(:,2)>0.63*K,1);
        T = slice(tT);
        PT1(i,:) = [K T];
        
        PT1fit(:,2*i-1:2*i) = [slice(:,1)+offsetT K*(1-exp(-(slice(:,1)/T)))];
    end
    
    PT1
        
    subplot(2, 2, 2);
    plot(sampleTimes, in.mot.vel(:,1), sampleTimes, out.mot.vol(:,1)*uF, ...
        PT1fit(:,1), PT1fit(:,2));
    legend('Real', 'Setpoint', 'PT1');
    title('M1 (Front Right)');
    ylabel('v [Motor RPS]');
    xlabel('t [s]');
    axis tight;
	ylim(vlim);    

    subplot(2, 2, 1);
    plot(sampleTimes, in.mot.vel(:,2), sampleTimes, out.mot.vol(:,2)*uF, ...
        PT1fit(:,3), PT1fit(:,4));
    title('M2 (Front Left)');
    ylabel('v [Motor RPS]');
    xlabel('t [s]');
	axis tight
	ylim(vlim);

    subplot(2, 2, 3);
    plot(sampleTimes, in.mot.vel(:,3), sampleTimes, out.mot.vol(:,3)*uF, ...
        PT1fit(:,5), PT1fit(:,6));
    title('M3 (Rear Left)');
    ylabel('v [Motor RPS]');
    xlabel('t [s]');
	axis tight
	ylim(vlim);

    subplot(2, 2, 4);
    plot(sampleTimes, in.mot.vel(:,4), sampleTimes, out.mot.vol(:,4)*uF, ...
        PT1fit(:,7), PT1fit(:,8));
    title('M4 (Rear Right)');
    ylabel('v [Motor RPS]');
    xlabel('t [s]');
	axis tight
	ylim(vlim);
end

function showGyrFFT(~, ~)
    subplot(2, 1, 1);
    plot(logData.gyro_raw.data(:,1), logData.gyro_raw.data(:,4))
    title('Gyro Raw Data');
    xlabel('t [s]');
    ylabel('Raw');
	axis tight

    g = logData.gyro_raw.data;

    Fs = 800;
    L = length(g);

    NFFT = 2^nextpow2(L);
    Yx = fft(g(:,4), NFFT)/L;
    f = Fs/2*linspace(0,1,NFFT/2+1);

    absYx = 2*abs(Yx(1:NFFT/2+1));

    subplot(2, 1, 2);
    plot(f(10:end), absYx(10:end));
    title('FFT Analysis');
    xlabel('F [Hz]');
    ylabel('Amplitude');
	axis tight
end

function showAccFFT(~, ~)
    subplot(2, 1, 1);
    plot(logData.acc_raw.data(:,1), logData.acc_raw.data(:,4))
    title('Accelerometer Raw Data');
    xlabel('t [s]');
    ylabel('Raw');
	axis tight

    g = logData.acc_raw.data;

    Fs = 800;
    L = length(g);

    NFFT = 2^nextpow2(L);
    Yx = fft(g(:,4), NFFT)/L;
    f = Fs/2*linspace(0,1,NFFT/2+1);

    absYx = 2*abs(Yx(1:NFFT/2+1));

    subplot(2, 1, 2);
    plot(f(10:end), absYx(10:end));
    title('FFT Analysis');
    xlabel('F [Hz]');
    ylabel('Amplitude');
	axis tight
end

function showLocalVelocity(~, ~)
	subplot(2, 1, 1);
	plot(sampleTimes, in.vel.enc.local(:,1), 'b-', sampleTimes, out.vel.enc.local(:,1), 'c:', ...
		sampleTimes, in.vel.enc.local(:,2), 'r-', sampleTimes, out.vel.enc.local(:,2), 'm:', ...
		sampleTimes, in.vel.acc.local(:,1), 'g-', sampleTimes, in.vel.acc.local(:,2), 'k-', ...
        sampleTimes, in.vel.vis.local(:,1), 'g--', sampleTimes, in.vel.vis.local(:,2), 'k--');
	legend('X Enc', 'X Setpoint', 'Y Enc', 'Y Setpoint', 'X Acc', 'Y Acc', 'X Vision', 'Y Vision');
	title('XY Velocities (Encoders+Accelerometer+Vision)');
	xlabel('t [s]');
	ylabel('v [m/s]');
	axis tight

	subplot(2, 1, 2);
	plot(sampleTimes, in.vel.enc.local(:,3), 'g-', ...
		sampleTimes, in.vel.gyr.local(:,3), 'm-', ...
		sampleTimes, out.vel.enc.local(:,3), 'k:', ...
        sampleTimes, in.vel.vis.local(:,3), 'c--');
	legend('W Enc', 'W Gyro', 'W Setpoint', 'W Vision');
	title('Rotational Velocity (Encoders+Gyro+Vision)');
	xlabel('t [s]');
	ylabel('v [rad/s]');
	axis tight
end

function showLocalVelocity2(~, ~)
	subplot(2, 2, 1);
	plot(sampleTimes, in.vel.enc.local(:,1), 'g--', ...
		sampleTimes, in.vel.acc.local(:,1), 'm--', ...
        sampleTimes, in.vel.vis.local(:,1), 'c-', ...
        sampleTimes, state.vel.local(:,1), 'b-');
	legend('Enc', 'Acc', 'Vision', 'State');
	title('X Velocities (Encoders+Accelerometer+Vision)');
	xlabel('t [s]');
	ylabel('v [m/s]');
	axis tight
    
	subplot(2, 2, 2);
	plot(sampleTimes, in.vel.enc.local(:,2), 'g--', ...
		sampleTimes, in.vel.acc.local(:,2), 'm--', ...
        sampleTimes, in.vel.vis.local(:,2), 'c-', ...
        sampleTimes, state.vel.local(:,2), 'b-');
    legend('Enc', 'Acc', 'Vision', 'State');
	title('Y Velocities (Encoders+Accelerometer+Vision)');
	xlabel('t [s]');
	ylabel('v [m/s]');
	axis tight

	subplot(2, 2, 3);
	plot(sampleTimes, in.vel.enc.local(:,3), 'g--', ...
		sampleTimes, in.vel.gyr.local(:,3), 'm--', ...
        sampleTimes, in.vel.vis.local(:,3), 'c-', ...
        sampleTimes, state.vel.local(:,3), 'b-');
	legend('Enc', 'Gyro', 'Vision', 'State');
	title('Rotational Velocity (Encoders+Gyro+Vision)');
	xlabel('t [s]');
	ylabel('v [rad/s]');
	axis tight

	subplot(2, 2, 4);
	plot(sampleTimes, in.vel.enc.local(:,4), 'g--', ...
		sampleTimes, in.vel.acc.local(:,4), 'm--', ...
        sampleTimes, in.vel.vis.local(:,4), 'c-', ...
        sampleTimes, state.vel.local(:,4), 'b-');
	legend('Enc', 'Acc', 'Vision', 'State');
	title('Norm(XY) Velocity (Encoders+Gyro+Vision)');
	xlabel('t [s]');
	ylabel('v [m/s]');
	axis tight
end

function showGlobalPosition(~, ~)
    subplot(1, 1, 1);
    plot(in.pos.vis.global(:,1), in.pos.vis.global(:,2), ...
		in.pos.vis.global(1,1), in.pos.vis.global(1,2), 'x');
	axis equal
    legend('Vision');
    title('Global Position (Vision)');
    xlabel('X [m]');
    ylabel('Y [m]');
end

function showAcceleration(~, ~)
    subplot(1, 1, 1);
    plot(sampleTimes, in.acc.acc.local(:,1), 'b-', sampleTimes, in.acc.acc.local(:,2), 'r-', ...
		sampleTimes, in.acc.vis.local(:,1), 'c:', sampleTimes, in.acc.vis.local(:,2), 'm:');
    axis tight
    legend('X Acc', 'Y Acc', 'X Vision', 'Y Vision');
    title('Acceleration');
    xlabel('t [s]');
    ylabel('a [m/s^2]');
end

function showAccelerometerVelocity(~, ~)
    subplot(1, 1, 1);
    plot(sampleTimes, in.vel.acc.local(:,1), sampleTimes, in.vel.acc.local(:,2));
    axis tight
    legend('X', 'Y');
    title('Integrated Accelerometer Velocity');
    xlabel('t [s]');
    ylabel('v [m/s]');
end

function showPID(~, ~)
    subplot(2, 3, 1);
    plot(pid.t, [pid.pid_pos_x_err+pid.pid_pos_x_in, pid.pid_pos_x_in, pid.pid_pos_x_err, pid.pid_pos_x_out]);
    axis tight
    legend('setpoint', 'in', 'err', 'out');
    title('PID pos x');
    xlabel('t [s]');
    ylabel('pos delta [m]');
    
    subplot(2, 3, 2);
    plot(pid.t, [pid.pid_pos_y_err+pid.pid_pos_y_in, pid.pid_pos_y_in, pid.pid_pos_y_err, pid.pid_pos_y_out]);
    axis tight
    legend('setpoint', 'in', 'err', 'out');
    title('PID pos y');
    xlabel('t [s]');
    ylabel('pos delta [m]');
    
    subplot(2, 3, 3);
    plot(pid.t, [pid.pid_pos_z_err+pid.pid_pos_z_in, pid.pid_pos_z_in, pid.pid_pos_z_err, pid.pid_pos_z_out]);
    axis tight
    legend('setpoint', 'in', 'err', 'out');
    title('PID pos w');
    xlabel('t [s]');
    ylabel('pos delta [m]');
    
    subplot(2, 3, 4);
    plot(pid.t, [pid.pid_vel_x_err+pid.pid_vel_x_in, pid.pid_vel_x_in, pid.pid_vel_x_err, pid.pid_vel_x_out]);
    axis tight
    legend('setpoint', 'in', 'err', 'out');
    title('PID vel x');
    xlabel('t [s]');
    ylabel('vel [m/s]');
    
    subplot(2, 3, 5);
    plot(pid.t, [pid.pid_vel_y_err+pid.pid_vel_y_in, pid.pid_vel_y_in, pid.pid_vel_y_err, pid.pid_vel_y_out]);
    axis tight
    legend('setpoint', 'in', 'err', 'out');
    title('PID vel y');
    xlabel('t [s]');
    ylabel('vel [m/s]');
    
    subplot(2, 3, 6);
    plot(pid.t, [pid.pid_vel_z_err+pid.pid_vel_z_in, pid.pid_vel_z_in, pid.pid_vel_z_err, pid.pid_vel_z_out]);
    axis tight
    legend('setpoint', 'in', 'err', 'out');
    title('PID vel w');
    xlabel('t [s]');
    ylabel('vel [m/s]');
end

function showTrajectory(~, ~)
  
    newTraj = find(pid.newTrajTime(2:end) > pid.newTrajTime(1:end-1));
    newTrajX = zeros(size(newTraj,1)*2,1);
    newTrajY = zeros(size(newTraj,1)*2,1);
    newTrajX(1:2:end) = pid.t(newTraj);
    newTrajX(2:2:end) = pid.t(newTraj);
    newTrajY(1:4:end) = -3;
    newTrajY(2:4:end) = 3;
    newTrajY(3:4:end) = 3;
    newTrajY(4:4:end) = -3;
    
    subplot(2,2,1);
    hold all;
    plot(pid.t, [pid.traj_pos_x, pid.traj_pos_y, pid.traj_pos_z]);
    plot(sampleTimes, state.pos.global);
    plot(newTrajX, newTrajY,'k');
    axis tight
    legend('x traj','y traj','z traj', 'x state', 'y state', 'z state');
    title('Trajectory pos');
    xlabel('t [s]');
    ylabel('pos [m]');
    
    subplot(2,2,3);
    hold all;
    plot(pid.t, [pid.traj_vel_x, pid.traj_vel_y, pid.traj_vel_z]);
    plot(sampleTimes, state.vel.global(:,1:3));
    plot(newTrajX, newTrajY,'k');
    axis tight
    legend('x traj','y traj','z traj', 'x state', 'y state', 'z state');
    title('Trajectory vel');
    xlabel('t [s]');
    ylabel('vel [m/s]');
    
    subplot(2,2,2);
    hold all;
    plot(pid.t, [pid.traj_acc_x, pid.traj_acc_y, pid.traj_acc_z]);
    plot(sampleTimes, state.acc.global(:,1:3));
    plot(newTrajX, newTrajY,'k');
    axis tight
    legend('x traj','y traj','z traj', 'x state', 'y state', 'z state');
    title('Trajectory acc');
    xlabel('t [s]');
    ylabel('acc [m/s^2]');
    
    subplot(2,2,4);
    hold all;
    plot(pid.t, [pid.acc_fwd_x, pid.acc_fwd_y, pid.acc_fwd_z]);
    plot(newTrajX, newTrajY,'k');
    axis tight
    legend('x','y','z');
    title('acc feedforward');
    xlabel('t [s]');
    ylabel('acc [m/s^2]');
    
    hold off;
end

end

